@@ -37,6 +37,7 @@ public:
     void setCore(Editor* e) { mEditor = e; }
 
     // file
+    Status importAnimatedImage();
     Status importMovieVideo();
     Status importSound(FileType type);
     Status exportMovie(bool isGif = false);
@@ -63,10 +64,18 @@ public:
     void GotoNextKeyFrame();
     void GotoPrevKeyFrame();
     Status addNewKey();
+
+    /** Will insert a keyframe at the current position and push connected frames to the right */
+    Status insertKeyFrameAtCurrentPosition();
     void removeKey();
+    void duplicateLayer();
     void duplicateKey();
     void moveFrameForward();
     void moveFrameBackward();
+    void removeSelectedFrames();
+    void reverseSelectedFrames();
+    void addExposureToSelectedFrames();
+    void subtractExposureFromSelectedFrames();
 
     // Layer
     Status addNewBitmapLayer();
@@ -91,11 +100,16 @@ public:
     void about();
 
 private:
+    void showSoundClipWarningIfNeeded();
+
+    void exposeSelectedFrames(int offset);
 
     Status convertSoundToWav(const QString& filePath);
 
     Editor* mEditor = nullptr;
     QWidget* mParent = nullptr;
+
+    bool mSuppressSoundWarning = false;
 };
 
 #endif // COMMANDCENTER_H
