@@ -31,7 +31,17 @@ ExportMovieDialog::ExportMovieDialog(QWidget *parent, Mode mode, FileType fileTy
     } else {
         setWindowTitle(tr("Export Movie"));
     }
+
+    QSizePolicy policy = ui->unevenWidthLabel->sizePolicy();
+    policy.setRetainSizeWhenHidden(true);
+    ui->unevenWidthLabel->setSizePolicy(policy);
+    policy = ui->unevenHeightLabel->sizePolicy();
+    policy.setRetainSizeWhenHidden(true);
+    ui->unevenHeightLabel->setSizePolicy(policy);
+
     connect(this, &ExportMovieDialog::filePathsChanged, this, &ExportMovieDialog::onFilePathsChanged);
+    connect(ui->widthSpinBox, static_cast<void(QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &ExportMovieDialog::validateResolution);
+    connect(ui->heightSpinBox, static_cast<void(QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &ExportMovieDialog::validateResolution);
 }
 
 ExportMovieDialog::~ExportMovieDialog()
@@ -66,6 +76,7 @@ void ExportMovieDialog::updateResolutionCombo( int index )
 
     ui->widthSpinBox->setValue( camSize.width() );
     ui->heightSpinBox->setValue( camSize.height() );
+    validateResolution();
 }
 
 void ExportMovieDialog::setDefaultRange(int startFrame, int endFrame, int endFrameWithSounds)
@@ -128,6 +139,7 @@ void ExportMovieDialog::onFilePathsChanged(QStringList filePaths)
         ui->loopCheckBox->setChecked(false);
     }
     ui->transparencyCheckBox->setEnabled(supportsTransparency(filePath));
+    validateResolution();
 }
 
 bool ExportMovieDialog::supportsLooping(QString filePath) const
@@ -141,3 +153,13 @@ bool ExportMovieDialog::supportsTransparency(QString filePath) const
     return filePath.endsWith(".apng", Qt::CaseInsensitive) ||
            filePath.endsWith(".webm", Qt::CaseInsensitive);
 }
+
+void ExportMovieDialog::validateResolution()
+{
+    const bool isMp4 = getFilePath().endsWith(".mp4", Qt::CaseInsensitive);
+    const bool widthValid = !isMp4 || ui->widthSpinBox->value() % 2 == 0;
+    const bool heightValid = !isMp4 || ui->heightSpinBox->value() % 2 == 0;
+    ui->unevenWidthLabel->setHidden(widthValid);
+    ui->unevenHeightLabel->setHidden(heightValid);
+    setOkButtonEnabled(widthValid && heightValid);
+}
