@@ -19,6 +19,7 @@ GNU General Public License for more details.
 
 #include "preferencemanager.h"
 #include "editor.h"
+#include "flowlayout.h"
 #include "util.h"
 
 OnionSkinWidget::OnionSkinWidget(QWidget *parent) :
@@ -26,6 +27,7 @@ OnionSkinWidget::OnionSkinWidget(QWidget *parent) :
     ui(new Ui::OnionSkin)
 {
     ui->setupUi(this);
+
     clearFocusOnFinished(ui->onionPrevFramesNumBox);
     clearFocusOnFinished(ui->onionNextFramesNumBox);
     clearFocusOnFinished(ui->onionMinOpacityBox);
@@ -42,10 +44,36 @@ void OnionSkinWidget::initUI()
     updateUI();
     makeConnections();
 
+    // Change the horizontal layout in the Distributed Opacity group box to a
+    // flow layout to reduce the minimum width
+    FlowLayout *opacityLayout = new FlowLayout;
+    opacityLayout->setAlignment(Qt::AlignHCenter);
+    opacityLayout->setContentsMargins(0, 6, 0, 6);
+    ui->opacityGroup->layout()->removeWidget(ui->minOpacityGroup);
+    ui->opacityGroup->layout()->removeWidget(ui->maxOpacityGroup);
+    opacityLayout->addWidget(ui->minOpacityGroup);
+    opacityLayout->addWidget(ui->maxOpacityGroup);
+    delete ui->opacityGroup->layout();
+    ui->opacityGroup->setLayout(opacityLayout);
+
 #ifdef __APPLE__
+
+    ui->scrollAreaWidgetContents->layout()->setSpacing(8);
+
     // Mac only style. ToolButtons are naturally borderless on Win/Linux.
     QString stylesheet =
         "QToolButton { border: 0px; } "
+        "QGroupBox::title {"
+            "subcontrol-origin: padding;"
+            "left: 6px;"
+            "padding: 2px 2px 0px 0px;"
+            "background: transparent;"
+        "}"
+        "QGroupBox {"
+            "subcontrol-origin: margin;"
+            "margin-top: 8px;"
+            "padding-top: 16px;"
+        "}"
         "QToolButton:pressed{ border: 1px solid #FFADAD; border-radius: 2px; background-color: #D5D5D5; }"
         "QToolButton:checked{ border: 1px solid #ADADAD; border-radius: 2px; background-color: #D5D5D5; }";
     setStyleSheet(this->styleSheet().append(stylesheet));
@@ -60,8 +88,8 @@ void OnionSkinWidget::makeConnections()
     connect(ui->onionPrevFramesNumBox, spinBoxChanged, this, &OnionSkinWidget::onionPrevFramesNumChange);
     connect(ui->onionNextFramesNumBox, spinBoxChanged, this, &OnionSkinWidget::onionNextFramesNumChange);
 
-    connect(ui->onionPrevButton, &QToolButton::clicked, this, &OnionSkinWidget::onionPrevButtonClicked);
-    connect(ui->onionNextButton, &QToolButton::clicked, this, &OnionSkinWidget::onionNextButtonClicked);
+    connect(ui->prevFramesGroup, &QGroupBox::clicked, this, &OnionSkinWidget::prevFramesGroupClicked);
+    connect(ui->nextFramesGroup, &QGroupBox::clicked, this, &OnionSkinWidget::nextFramesGroupClicked);
     connect(ui->onionBlueButton, &QToolButton::clicked, this, &OnionSkinWidget::onionBlueButtonClicked);
     connect(ui->onionRedButton, &QToolButton::clicked, this, &OnionSkinWidget::onionRedButtonClicked);
 
@@ -77,18 +105,12 @@ void OnionSkinWidget::updateUI()
 {
     PreferenceManager* prefs = editor()->preference();
 
-    QSignalBlocker b1(ui->onionPrevButton);
-    ui->onionPrevButton->setChecked(prefs->isOn(SETTING::PREV_ONION));
-
-    QSignalBlocker b2(ui->onionNextButton);
-    ui->onionNextButton->setChecked(prefs->isOn(SETTING::NEXT_ONION));
+    ui->prevFramesGroup->setChecked(prefs->isOn(SETTING::PREV_ONION));
+    ui->nextFramesGroup->setChecked(prefs->isOn(SETTING::NEXT_ONION));
 
     QSignalBlocker b3(ui->onionBlueButton);
     ui->onionBlueButton->setChecked(prefs->isOn(SETTING::ONION_BLUE));
 
-    ui->onionRedButton->setEnabled(ui->onionPrevButton->isChecked());
-    ui->onionBlueButton->setEnabled(ui->onionNextButton->isChecked());
-
     QSignalBlocker b4(ui->onionRedButton);
     ui->onionRedButton->setChecked(prefs->isOn(SETTING::ONION_RED));
 
@@ -105,13 +127,13 @@ void OnionSkinWidget::updateUI()
 
 }
 
-void OnionSkinWidget::onionPrevButtonClicked(bool isOn)
+void OnionSkinWidget::prevFramesGroupClicked(bool isOn)
 {
     PreferenceManager* prefs = editor()->preference();
     prefs->set(SETTING::PREV_ONION, isOn);
 }
 
-void OnionSkinWidget::onionNextButtonClicked(bool isOn)
+void OnionSkinWidget::nextFramesGroupClicked(bool isOn)
 {
     PreferenceManager* prefs = editor()->preference();
     prefs->set(SETTING::NEXT_ONION, isOn);
