@@ -44,6 +44,8 @@ public:
     void updateUI() override;
 
 public slots:
+    void onToolSetActive(ToolType toolType);
+    void onLayerDidChange(int index);
     void pencilOn();
     void eraserOn();
     void selectOn();
@@ -59,12 +61,9 @@ public slots:
 protected:
     int getMinHeightForWidth(int width) override;
 
-signals:
-    void clearButtonClicked();
-
 private:
     void deselectAllTools();
-    bool leavingTool(QToolButton*);
+    void toolOn(ToolType toolType, QToolButton* toolButton);
 
     Ui::ToolBoxWidget* ui = nullptr;
 };
