@@ -20,11 +20,14 @@ GNU General Public License for more details.
 #include <QSettings>
 #include <QDebug>
 
+#include "cameraoptionswidget.h"
+#include "bucketoptionswidget.h"
 #include "spinslider.h"
 #include "editor.h"
 #include "util.h"
 #include "layer.h"
 #include "layermanager.h"
+#include "stroketool.h"
 #include "toolmanager.h"
 
 ToolOptionWidget::ToolOptionWidget(QWidget* parent) : BaseDockWidget(parent)
@@ -44,19 +47,20 @@ ToolOptionWidget::~ToolOptionWidget()
 
 void ToolOptionWidget::initUI()
 {
+    mBucketOptionsWidget = new BucketOptionsWidget(editor(), this);
+    mCameraOptionsWidget = new CameraOptionsWidget(editor(), this);
+    ui->horizontalLayout_2->addWidget(mBucketOptionsWidget);
+    ui->horizontalLayout_2->addWidget(mCameraOptionsWidget);
+
     QSettings settings(PENCIL2D, PENCIL2D);
 
-    ui->sizeSlider->init(tr("Width"), SpinSlider::EXPONENT, SpinSlider::INTEGER, 1, 200);
+    ui->sizeSlider->init(tr("Width"), SpinSlider::EXPONENT, SpinSlider::INTEGER, StrokeTool::WIDTH_MIN, StrokeTool::WIDTH_MAX);
     ui->sizeSlider->setValue(settings.value("brushWidth", "3").toDouble());
     ui->brushSpinBox->setValue(settings.value("brushWidth", "3").toDouble());
 
-    ui->featherSlider->init(tr("Feather"), SpinSlider::LOG, SpinSlider::INTEGER, 1, 99);
+    ui->featherSlider->init(tr("Feather"), SpinSlider::LOG, SpinSlider::INTEGER, StrokeTool::FEATHER_MIN, StrokeTool::FEATHER_MAX);
     ui->featherSlider->setValue(settings.value("brushFeather", "5").toDouble());
     ui->featherSpinBox->setValue(settings.value("brushFeather", "5").toDouble());
-
-    ui->toleranceSlider->init(tr("Color Tolerance"), SpinSlider::LINEAR, SpinSlider::INTEGER, 0, 100);
-    ui->toleranceSlider->setValue(settings.value("Tolerance", "50").toInt());
-    ui->toleranceSpinBox->setValue(settings.value("Tolerance", "50").toInt());
 }
 
 void ToolOptionWidget::updateUI()
@@ -64,8 +68,6 @@ void ToolOptionWidget::updateUI()
     BaseTool* currentTool = editor()->tools()->currentTool();
     Q_ASSERT(currentTool);
 
-    disableAllOptions();
-
     setVisibility(currentTool);
 
     const Properties& p = currentTool->properties;
@@ -79,8 +81,8 @@ void ToolOptionWidget::updateUI()
     setVectorMergeEnabled(p.vectorMergeEnabled);
     setAA(p.useAA);
     setStabilizerLevel(p.stabilizerLevel);
-    setTolerance(static_cast<int>(p.tolerance));
     setFillContour(p.useFillContour);
+    setShowSelectionInfo(p.showSelectionInfo);
 }
 
 void ToolOptionWidget::createUI()
@@ -111,12 +113,10 @@ void ToolOptionWidget::makeConnectionToEditor(Editor* editor)
 
     connect(ui->inpolLevelsCombo, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated), toolManager, &ToolManager::setStabilizerLevel);
 
-    connect(ui->toleranceSlider, &SpinSlider::valueChanged, toolManager, &ToolManager::setTolerance);
-    connect(ui->toleranceSpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), toolManager, &ToolManager::setTolerance);
-    clearFocusOnFinished(ui->toleranceSpinBox);
-
     connect(ui->fillContourBox, &QCheckBox::clicked, toolManager, &ToolManager::setUseFillContour);
 
+    connect(ui->showInfoBox, &QCheckBox::clicked, toolManager, &ToolManager::setShowSelectionInfo);
+
     connect(toolManager, &ToolManager::toolChanged, this, &ToolOptionWidget::onToolChanged);
     connect(toolManager, &ToolManager::toolPropertyChanged, this, &ToolOptionWidget::onToolPropertyChanged);
 }
@@ -136,9 +136,16 @@ void ToolOptionWidget::onToolPropertyChanged(ToolType, ToolPropertyType ePropert
     case VECTORMERGE: setVectorMergeEnabled(p.vectorMergeEnabled); break;
     case ANTI_ALIASING: setAA(p.useAA); break;
     case STABILIZATION: setStabilizerLevel(p.stabilizerLevel); break;
-    case TOLERANCE: setTolerance(static_cast<int>(p.tolerance)); break;
     case FILLCONTOUR: setFillContour(p.useFillContour); break;
+    case SHOWSELECTIONINFO: setShowSelectionInfo(p.showSelectionInfo); break;
     case BEZIER: setBezier(p.bezier_state); break;
+    case CAMERAPATH: { break; }
+    case TOLERANCE: break;
+    case USETOLERANCE: break;
+    case BUCKETFILLEXPAND: break;
+    case USEBUCKETFILLEXPAND: break;
+    case BUCKETFILLLAYERREFERENCEMODE: break;
+    case FILL_MODE: break;
     default:
         Q_ASSERT(false);
         break;
@@ -147,6 +154,26 @@ void ToolOptionWidget::onToolPropertyChanged(ToolType, ToolPropertyType ePropert
 
 void ToolOptionWidget::setVisibility(BaseTool* tool)
 {
+    Q_ASSERT(mBucketOptionsWidget);
+    Q_ASSERT(mCameraOptionsWidget);
+
+    disableAllOptions();
+
+    if (tool->type() == BUCKET)
+    {
+        mBucketOptionsWidget->setHidden(false);
+        return;
+    }
+    else if (tool->type() == CAMERA)
+    {
+        mCameraOptionsWidget->setHidden(false);
+    }
+    else
+    {
+        mCameraOptionsWidget->setHidden(true);
+        mBucketOptionsWidget->setHidden(true);
+    }
+
     ui->sizeSlider->setVisible(tool->isPropertyEnabled(WIDTH));
     ui->brushSpinBox->setVisible(tool->isPropertyEnabled(WIDTH));
     ui->featherSlider->setVisible(tool->isPropertyEnabled(FEATHER));
@@ -159,9 +186,8 @@ void ToolOptionWidget::setVisibility(BaseTool* tool)
     ui->useAABox->setVisible(tool->isPropertyEnabled(ANTI_ALIASING));
     ui->stabilizerLabel->setVisible(tool->isPropertyEnabled(STABILIZATION));
     ui->inpolLevelsCombo->setVisible(tool->isPropertyEnabled(STABILIZATION));
-    ui->toleranceSlider->setVisible(tool->isPropertyEnabled(TOLERANCE));
-    ui->toleranceSpinBox->setVisible(tool->isPropertyEnabled(TOLERANCE));
     ui->fillContourBox->setVisible(tool->isPropertyEnabled(FILLCONTOUR));
+    ui->showInfoBox->setVisible(tool->isPropertyEnabled(SHOWSELECTIONINFO));
 
     auto currentLayerType = editor()->layers()->currentLayer()->type();
     auto propertyType = editor()->tools()->currentTool()->type();
@@ -183,15 +209,8 @@ void ToolOptionWidget::setVisibility(BaseTool* tool)
             ui->brushSpinBox->setVisible(false);
             ui->usePressureBox->setVisible(false);
             break;
-        case BUCKET:
-            ui->sizeSlider->setLabel(tr("Stroke Thickness"));
-            ui->toleranceSlider->setVisible(false);
-            ui->toleranceSpinBox->setVisible(false);
-            break;
         default:
             ui->sizeSlider->setLabel(tr("Width"));
-            ui->toleranceSlider->setVisible(false);
-            ui->toleranceSpinBox->setVisible(false);
             ui->useAABox->setVisible(false);
             break;
         }
@@ -207,6 +226,15 @@ void ToolOptionWidget::setVisibility(BaseTool* tool)
             ui->brushSpinBox->setVisible(false);
             ui->sizeSlider->setVisible(false);
             break;
+        case SELECT:
+        case MOVE:
+            ui->sizeSlider->setVisible(false);
+            ui->brushSpinBox->setVisible(false);
+            ui->usePressureBox->setVisible(false);
+            ui->featherSlider->setVisible(false);
+            ui->featherSpinBox->setVisible(false);
+            ui->useFeatherBox->setVisible(false);
+            break;
         default:
             ui->makeInvisibleBox->setVisible(false);
             break;
@@ -304,17 +332,6 @@ void ToolOptionWidget::setStabilizerLevel(int x)
     ui->inpolLevelsCombo->setCurrentIndex(qBound(0, x, ui->inpolLevelsCombo->count() - 1));
 }
 
-void ToolOptionWidget::setTolerance(int tolerance)
-{
-    QSignalBlocker b(ui->toleranceSlider);
-    ui->toleranceSlider->setEnabled(true);
-    ui->toleranceSlider->setValue(tolerance);
-
-    QSignalBlocker b2(ui->toleranceSpinBox);
-    ui->toleranceSpinBox->setEnabled(true);
-    ui->toleranceSpinBox->setValue(tolerance);
-}
-
 void ToolOptionWidget::setFillContour(int useFill)
 {
     QSignalBlocker b(ui->fillContourBox);
@@ -328,6 +345,12 @@ void ToolOptionWidget::setBezier(bool useBezier)
     ui->useBezierBox->setChecked(useBezier);
 }
 
+void ToolOptionWidget::setShowSelectionInfo(bool showSelectionInfo)
+{
+    QSignalBlocker b(ui->showInfoBox);
+    ui->showInfoBox->setChecked(showSelectionInfo);
+}
+
 void ToolOptionWidget::disableAllOptions()
 {
     ui->sizeSlider->hide();
@@ -342,7 +365,7 @@ void ToolOptionWidget::disableAllOptions()
     ui->vectorMergeBox->hide();
     ui->useAABox->hide();
     ui->inpolLevelsCombo->hide();
-    ui->toleranceSlider->hide();
-    ui->toleranceSpinBox->hide();
     ui->fillContourBox->hide();
+    ui->showInfoBox->hide();
+    ui->stabilizerLabel->hide();
 }
