@@ -39,7 +39,9 @@ namespace PlatformHandler
 
     void initialise()
     {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+#endif
     }
 }
 
@@ -65,13 +67,13 @@ void detectWhichOSX()
 
 void disableCoalescing()
 {
-    SetMouseCoalescingEnabled(gIsMouseCoalescing, NULL);
+    SetMouseCoalescingEnabled(gIsMouseCoalescing, nullptr);
     //MacOSXNative::setMouseCoalescingEnabled(false);
 }
 
 void enableCoalescing()
 {
-    SetMouseCoalescingEnabled(true, NULL);
+    SetMouseCoalescingEnabled(true, nullptr);
     //MacOSXNative::setMouseCoalescingEnabled(true);
 }
 
