@@ -18,15 +18,17 @@ GNU General Public License for more details.
 
 #include <cmath>
 #include <QDebug>
-#include <QtMath>
 #include <QFile>
+#include <QFileInfo>
 #include <QPainterPath>
 #include "util.h"
 
+#include "blitrect.h"
+#include "tile.h"
+#include "tiledbuffer.h"
+
 BitmapImage::BitmapImage()
 {
-    mImage.reset(new QImage); // create null image
-    mBounds = QRect(0, 0, 0, 0);
 }
 
 BitmapImage::BitmapImage(const BitmapImage& a) : KeyFrame(a)
@@ -34,14 +36,15 @@ BitmapImage::BitmapImage(const BitmapImage& a) : KeyFrame(a)
     mBounds = a.mBounds;
     mMinBound = a.mMinBound;
     mEnableAutoCrop = a.mEnableAutoCrop;
-    mImage.reset(new QImage(*a.mImage));
+    mOpacity = a.mOpacity;
+    mImage = a.mImage;
 }
 
 BitmapImage::BitmapImage(const QRect& rectangle, const QColor& color)
 {
     mBounds = rectangle;
-    mImage.reset(new QImage(mBounds.size(), QImage::Format_ARGB32_Premultiplied));
-    mImage->fill(color.rgba());
+    mImage = QImage(mBounds.size(), QImage::Format_ARGB32_Premultiplied);
+    mImage.fill(color.rgba());
     mMinBound = false;
 }
 
@@ -49,15 +52,15 @@ BitmapImage::BitmapImage(const QPoint& topLeft, const QImage& image)
 {
     mBounds = QRect(topLeft, image.size());
     mMinBound = true;
-    mImage.reset(new QImage(image));
+    mImage = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
 }
 
 BitmapImage::BitmapImage(const QPoint& topLeft, const QString& path)
 {
     setFileName(path);
-    mImage.reset();
+    mImage = QImage();
 
-    mBounds = QRect(topLeft, QSize(0, 0));
+    mBounds = QRect(topLeft, QSize(-1, 0));
     mMinBound = true;
     setModified(false);
 }
@@ -68,8 +71,8 @@ BitmapImage::~BitmapImage()
 
 void BitmapImage::setImage(QImage* img)
 {
-    Q_CHECK_PTR(img);
-    mImage.reset(img);
+    Q_ASSERT(img && img->format() == QImage::Format_ARGB32_Premultiplied);
+    mImage = *img;
     mMinBound = false;
 
     modification();
@@ -85,22 +88,46 @@ BitmapImage& BitmapImage::operator=(const BitmapImage& a)
     KeyFrame::operator=(a);
     mBounds = a.mBounds;
     mMinBound = a.mMinBound;
-    mImage.reset(new QImage(*a.mImage));
+    mOpacity = a.mOpacity;
+    mImage = a.mImage;
     modification();
     return *this;
 }
 
-BitmapImage* BitmapImage::clone()
+BitmapImage* BitmapImage::clone() const
 {
-    return new BitmapImage(*this);
+    BitmapImage* b = new BitmapImage(*this);
+    b->setFileName(""); // don't link to the file of the source bitmap image
+
+    const bool validKeyFrame = !fileName().isEmpty();
+    if (validKeyFrame && !isLoaded())
+    {
+        // This bitmapImage is temporarily unloaded.
+        // since it's not in the memory, we need to copy the linked png file to prevent data loss.
+        QFileInfo finfo(fileName());
+        Q_ASSERT(finfo.isAbsolute());
+        Q_ASSERT(QFile::exists(fileName()));
+
+        QString newFileName = QString("%1/%2-%3.%4")
+            .arg(finfo.canonicalPath())
+            .arg(finfo.completeBaseName())
+            .arg(uniqueString(12))
+            .arg(finfo.suffix());
+        b->setFileName(newFileName);
+
+        bool ok = QFile::copy(fileName(), newFileName);
+        Q_ASSERT(ok);
+        qDebug() << "COPY>" << fileName();
+    }
+    return b;
 }
 
 void BitmapImage::loadFile()
 {
-    if (mImage == nullptr)
+    if (!fileName().isEmpty() && !isLoaded())
     {
-        mImage.reset(new QImage(fileName()));
-        mBounds.setSize(mImage->size());
+        mImage = QImage(fileName()).convertToFormat(QImage::Format_ARGB32_Premultiplied);
+        mBounds.setSize(mImage.size());
         mMinBound = false;
     }
 }
@@ -109,20 +136,20 @@ void BitmapImage::unloadFile()
 {
     if (isModified() == false)
     {
-        mImage.reset();
+        mImage = QImage();
     }
 }
 
-bool BitmapImage::isLoaded()
+bool BitmapImage::isLoaded() const
 {
-    return (mImage != nullptr);
+    return mImage.width() == mBounds.width();
 }
 
 quint64 BitmapImage::memoryUsage()
 {
-    if (mImage)
+    if (!mImage.isNull())
     {
-        return imageSize(*mImage);
+        return imageSize(mImage);
     }
     return 0;
 }
@@ -142,7 +169,7 @@ void BitmapImage::paintImage(QPainter& painter, QImage& image, QRect sourceRect,
 QImage* BitmapImage::image()
 {
     loadFile();
-    return mImage.get();
+    return &mImage;
 }
 
 BitmapImage BitmapImage::copy()
@@ -155,6 +182,7 @@ BitmapImage BitmapImage::copy(QRect rectangle)
     if (rectangle.isEmpty() || mBounds.isEmpty()) return BitmapImage();
 
     QRect intersection2 = rectangle.translated(-mBounds.topLeft());
+
     BitmapImage result(rectangle.topLeft(), image()->copy(intersection2));
     return result;
 }
@@ -178,6 +206,28 @@ void BitmapImage::paste(BitmapImage* bitmapImage, QPainter::CompositionMode cm)
     modification();
 }
 
+void BitmapImage::paste(const TiledBuffer* tiledBuffer, QPainter::CompositionMode cm)
+{
+    if(tiledBuffer->bounds().width() <= 0 || tiledBuffer->bounds().height() <= 0)
+    {
+        return;
+    }
+    extend(tiledBuffer->bounds());
+
+    QPainter painter(image());
+
+    painter.setCompositionMode(cm);
+    auto const tiles = tiledBuffer->tiles();
+    for (const Tile* item : tiles) {
+        const QPixmap& tilePixmap = item->pixmap();
+        const QPoint& tilePos = item->pos();
+        painter.drawPixmap(tilePos-mBounds.topLeft(), tilePixmap);
+    }
+    painter.end();
+
+    modification();
+}
+
 void BitmapImage::moveTopLeft(QPoint point)
 {
     mBounds.moveTopLeft(point);
@@ -189,16 +239,16 @@ void BitmapImage::transform(QRect newBoundaries, bool smoothTransform)
 {
     mBounds = newBoundaries;
     newBoundaries.moveTopLeft(QPoint(0, 0));
-    QImage* newImage = new QImage(mBounds.size(), QImage::Format_ARGB32_Premultiplied);
+    QImage newImage(mBounds.size(), QImage::Format_ARGB32_Premultiplied);
 
-    QPainter painter(newImage);
+    QPainter painter(&newImage);
     painter.setRenderHint(QPainter::SmoothPixmapTransform, smoothTransform);
     painter.setCompositionMode(QPainter::CompositionMode_Source);
-    painter.fillRect(newImage->rect(), QColor(0, 0, 0, 0));
+    painter.fillRect(newImage.rect(), QColor(0, 0, 0, 0));
     painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
     painter.drawImage(newBoundaries, *image());
     painter.end();
-    mImage.reset(newImage);
+    mImage = newImage;
 
     modification();
 }
@@ -245,15 +295,15 @@ void BitmapImage::updateBounds(QRect newBoundaries)
     // Check to make sure changes actually need to be made
     if (mBounds == newBoundaries) return;
 
-    QImage* newImage = new QImage( newBoundaries.size(), QImage::Format_ARGB32_Premultiplied);
-    newImage->fill(Qt::transparent);
-    if (!newImage->isNull())
+    QImage newImage(newBoundaries.size(), QImage::Format_ARGB32_Premultiplied);
+    newImage.fill(Qt::transparent);
+    if (!newImage.isNull())
     {
-        QPainter painter(newImage);
-        painter.drawImage(mBounds.topLeft() - newBoundaries.topLeft(), *mImage);
+        QPainter painter(&newImage);
+        painter.drawImage(mBounds.topLeft() - newBoundaries.topLeft(), mImage);
         painter.end();
     }
-    mImage.reset( newImage );
+    mImage = newImage;
     mBounds = newBoundaries;
     mMinBound = false;
 
@@ -279,15 +329,15 @@ void BitmapImage::extend(QRect rectangle)
     else
     {
         QRect newBoundaries = mBounds.united(rectangle).normalized();
-        QImage* newImage = new QImage(newBoundaries.size(), QImage::Format_ARGB32_Premultiplied);
-        newImage->fill(Qt::transparent);
-        if (!newImage->isNull())
+        QImage newImage(newBoundaries.size(), QImage::Format_ARGB32_Premultiplied);
+        newImage.fill(Qt::transparent);
+        if (!newImage.isNull())
         {
-            QPainter painter(newImage);
+            QPainter painter(&newImage);
             painter.drawImage(mBounds.topLeft() - newBoundaries.topLeft(), *image());
             painter.end();
         }
-        mImage.reset(newImage);
+        mImage = newImage;
         mBounds = newBoundaries;
 
         modification();
@@ -377,26 +427,26 @@ void BitmapImage::autoCrop()
 {
     if (!mEnableAutoCrop) return;
     if (mBounds.isEmpty()) return; // Exit if current bounds are null
-    if (!mImage) return;
+    if (mImage.isNull()) return;
 
-    Q_ASSERT(mBounds.size() == mImage->size());
+    Q_ASSERT(mBounds.size() == mImage.size());
 
     // Exit if already min bounded
     if (mMinBound) return;
 
     // Get image properties
-    const int width = mImage->width();
+    const int width = mImage.width();
 
     // Relative top and bottom row indices (inclusive)
     int relTop = 0;
-    int relBottom = mBounds.height()-1;
+    int relBottom = mBounds.height() - 1;
 
     // Check top row
     bool isEmpty = true; // Used to track if a non-transparent pixel has been found
     while (isEmpty && relTop <= relBottom) // Loop through rows
     {
         // Point cursor to the first pixel in the current top row
-        const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage->constScanLine(relTop));
+        const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage.constScanLine(relTop));
         for (int col = 0; col < width; col++) // Loop through pixels in row
         {
             // If the pixel is not transparent
@@ -424,7 +474,7 @@ void BitmapImage::autoCrop()
     while (isEmpty && relBottom >= relTop) // Loop through rows
     {
         // Point cursor to the first pixel in the current bottom row
-        const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage->constScanLine(relBottom));
+        const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage.constScanLine(relBottom));
         for (int col = 0; col < width; col++) // Loop through pixels in row
         {
             // If the pixel is not transparent
@@ -456,7 +506,7 @@ void BitmapImage::autoCrop()
     while (isEmpty && relBottom >= relTop && relLeft <= relRight) // Loop through columns
     {
         // Point cursor to the pixel at row relTop and column relLeft
-        const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage->constScanLine(relTop)) + relLeft;
+        const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage.constScanLine(relTop)) + relLeft;
         // Loop through pixels in column
         // Note: we only need to loop from relTop to relBottom (inclusive)
         //       not the full image height, because rows 0 to relTop-1 and
@@ -490,7 +540,7 @@ void BitmapImage::autoCrop()
     while (isEmpty && relRight >= relLeft) // Loop through columns
     {
         // Point cursor to the pixel at row relTop and column relRight
-        const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage->constScanLine(relTop)) + relRight;
+        const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage.constScanLine(relTop)) + relRight;
         // Loop through pixels in column
         // Note: we only need to loop from relTop to relBottom (inclusive)
         //       not the full image height, because rows 0 to relTop-1 and
@@ -598,7 +648,10 @@ void BitmapImage::drawRect(QRectF rectangle, QPen pen, QBrush brush, QPainter::C
         painter.setRenderHint(QPainter::Antialiasing, antialiasing);
         painter.setPen(pen);
         painter.setBrush(brush);
-        painter.drawRect(rectangle.translated(-mBounds.topLeft()));
+
+        // Adjust the brush rectangle to be bigger than the bounds itself,
+        // otherwise there will be artifacts shown in some cases when smudging
+        painter.drawRect(rectangle.translated(-mBounds.topLeft()).adjusted(-1, -1, 1, 1));
         painter.end();
     }
     modification();
@@ -675,59 +728,11 @@ void BitmapImage::drawPath(QPainterPath path, QPen pen, QBrush brush,
     modification();
 }
 
-PegbarResult BitmapImage::findLeft(QRectF rect, int grayValue)
-{
-    PegbarResult result;
-    result.value = -1;
-    result.errorcode = Status::FAIL;
-    int left = static_cast<int>(rect.left());
-    int right = static_cast<int>(rect.right());
-    int top = static_cast<int>(rect.top());
-    int bottom = static_cast<int>(rect.bottom());
-    for (int x = left; x <= right; x++)
-    {
-        for (int y = top; y <= bottom; y++)
-        {
-            if (qAlpha(constScanLine(x,y)) == 255 && qGray(constScanLine(x,y)) < grayValue)
-            {
-                result.value = x;
-                result.errorcode = Status::OK;
-                return result;
-            }
-        }
-    }
-    return result;
-}
-
-PegbarResult BitmapImage::findTop(QRectF rect, int grayValue)
-{
-    PegbarResult result;
-    result.value = -1;
-    result.errorcode = Status::FAIL;
-    int left = static_cast<int>(rect.left());
-    int right = static_cast<int>(rect.right());
-    int top = static_cast<int>(rect.top());
-    int bottom = static_cast<int>(rect.bottom());
-    for (int y = top; y <= bottom; y++)
-    {
-        for (int x = left; x <= right; x++)
-        {
-            if (qAlpha(constScanLine(x,y)) == 255 && qGray(constScanLine(x,y)) < grayValue)
-            {
-                result.value = y;
-                result.errorcode = Status::OK;
-                return result;
-            }
-        }
-    }
-    return result;
-}
-
 Status BitmapImage::writeFile(const QString& filename)
 {
-    if (mImage && !mImage->isNull())
+    if (!mImage.isNull())
     {
-        bool b = mImage->save(filename);
+        bool b = mImage.save(filename);
         return (b) ? Status::OK : Status::FAIL;
     }
 
@@ -746,7 +751,7 @@ Status BitmapImage::writeFile(const QString& filename)
 
 void BitmapImage::clear()
 {
-    mImage.reset(new QImage); // null image
+    mImage = QImage(); // null image
     mBounds = QRect(0, 0, 0, 0);
     mMinBound = true;
     modification();
@@ -754,27 +759,20 @@ void BitmapImage::clear()
 
 QRgb BitmapImage::constScanLine(int x, int y) const
 {
-    QRgb result = qRgba(0, 0, 0, 0);
-    if (mBounds.contains(QPoint(x, y)))
-    {
-        result = *(reinterpret_cast<const QRgb*>(mImage->constScanLine(y - mBounds.top())) + x - mBounds.left());
+    QRgb result = QRgb();
+    if (mBounds.contains(x, y)) {
+        result = *(reinterpret_cast<const QRgb*>(mImage.constScanLine(y - mBounds.top())) + x - mBounds.left());
     }
     return result;
 }
 
 void BitmapImage::scanLine(int x, int y, QRgb color)
 {
-    extend(QPoint(x, y));
-    if (mBounds.contains(QPoint(x, y)))
-    {
-        // Make sure color is premultiplied before calling
-        *(reinterpret_cast<QRgb*>(image()->scanLine(y - mBounds.top())) + x - mBounds.left()) =
-            qRgba(
-                qRed(color),
-                qGreen(color),
-                qBlue(color),
-                qAlpha(color));
+    if (!mBounds.contains(x, y)) {
+        return;
     }
+    // Make sure color is premultiplied before calling
+    *(reinterpret_cast<QRgb*>(image()->scanLine(y - mBounds.top())) + x - mBounds.left()) = color;
 }
 
 void BitmapImage::clear(QRect rectangle)
@@ -792,87 +790,124 @@ void BitmapImage::clear(QRect rectangle)
     modification();
 }
 
-/** Compare colors for the purposes of flood filling
- *
- *  Calculates the Eulcidian difference of the RGB channels
- *  of the image and compares it to the tolerance
- *
- *  @param[in] newColor The first color to compare
- *  @param[in] oldColor The second color to compare
- *  @param[in] tolerance The threshold limit between a matching and non-matching color
- *  @param[in,out] cache Contains a mapping of previous results of compareColor with rule that
- *                 cache[someColor] = compareColor(someColor, oldColor, tolerance)
- *
- *  @return Returns true if the colors have a similarity below the tolerance level
- *          (i.e. if Eulcidian distance squared is <= tolerance)
- */
-bool BitmapImage::compareColor(QRgb newColor, QRgb oldColor, int tolerance, QHash<QRgb, bool> *cache)
+bool BitmapImage::floodFill(BitmapImage** replaceImage,
+                            const BitmapImage* targetImage,
+                            const QRect& cameraRect,
+                            const QPoint& point,
+                            const QRgb& fillColor,
+                            int tolerance,
+                            const int expandValue)
 {
-    // Handle trivial case
-    if (newColor == oldColor) return true;
+    // Fill region must be 1 pixel larger than the target image to fill regions on the edge connected only by transparent pixels
+    const QRect& fillBounds = targetImage->mBounds;
+    QRect maxBounds = cameraRect.united(fillBounds).adjusted(-expandValue, -expandValue, expandValue, expandValue);
+    const int maxWidth = maxBounds.width(), left = maxBounds.left(), top = maxBounds.top();
 
-    if(cache && cache->contains(newColor)) return cache->value(newColor);
-
-    // Get Eulcidian distance between colors
-    // Not an accurate representation of human perception,
-    // but it's the best any image editing program ever does
-    int diffRed = static_cast<int>(qPow(qRed(oldColor) - qRed(newColor), 2));
-    int diffGreen = static_cast<int>(qPow(qGreen(oldColor) - qGreen(newColor), 2));
-    int diffBlue = static_cast<int>(qPow(qBlue(oldColor) - qBlue(newColor), 2));
-    // This may not be the best way to handle alpha since the other channels become less relevant as
-    // the alpha is reduces (ex. QColor(0,0,0,0) is the same as QColor(255,255,255,0))
-    int diffAlpha = static_cast<int>(qPow(qAlpha(oldColor) - qAlpha(newColor), 2));
-
-    bool isSimilar = (diffRed + diffGreen + diffBlue + diffAlpha) <= tolerance;
-
-    if(cache)
+    // If the point we are supposed to fill is outside the max bounds, do nothing
+    if(!maxBounds.contains(point))
     {
-        Q_ASSERT(cache->contains(isSimilar) ? isSimilar == (*cache)[newColor] : true);
-        (*cache)[newColor] = isSimilar;
-    }
-
-    return isSimilar;
-}
-
-// Flood fill
-// ----- http://lodev.org/cgtutor/floodfill.html
-void BitmapImage::floodFill(BitmapImage* targetImage,
-                            QRect cameraRect,
-                            QPoint point,
-                            QRgb newColor,
-                            int tolerance)
-{
-    // If the point we are supposed to fill is outside the image and camera bounds, do nothing
-    if(!cameraRect.united(targetImage->bounds()).contains(point))
-    {
-        return;
+        return false;
     }
 
     // Square tolerance for use with compareColor
     tolerance = static_cast<int>(qPow(tolerance, 2));
 
-    QRgb oldColor = targetImage->pixel(point);
+    QRect newBounds;
+    bool shouldFillBorder = false;
+    bool *filledPixels = floodFillPoints(targetImage, fillBounds, maxBounds, point, tolerance, newBounds, shouldFillBorder);
+
+    QRect translatedSearchBounds = newBounds.translated(-maxBounds.topLeft());
+
+    if (shouldFillBorder)
+    {
+        for (int y = 0; y < maxBounds.height(); y++)
+        {
+            for (int x = 0; x < maxBounds.width(); x++)
+            {
+                if(!translatedSearchBounds.contains(x, y))
+                {
+                    filledPixels[y*maxWidth+x] = true;
+                }
+            }
+        }
+        newBounds = maxBounds;
+    }
+
+    // The scanned bounds should take the expansion into account
+    const QRect& expandRect = newBounds.adjusted(-expandValue, -expandValue, expandValue, expandValue);
+    if (expandValue > 0) {
+        newBounds = expandRect;
+    }
+    if (!maxBounds.contains(newBounds)) {
+        newBounds = maxBounds;
+    }
+    translatedSearchBounds = newBounds.translated(-maxBounds.topLeft());
+
+    if (expandValue > 0) {
+        expandFill(filledPixels, translatedSearchBounds, maxBounds, expandValue);
+    }
+
+    *replaceImage = new BitmapImage(newBounds, Qt::transparent);
+
+    // Fill all the found pixels
+    for (int y = translatedSearchBounds.top(); y <= translatedSearchBounds.bottom(); y++)
+    {
+        for (int x = translatedSearchBounds.left(); x <= translatedSearchBounds.right(); x++)
+        {
+            const int index = y * maxWidth + x;
+            if (!filledPixels[index])
+            {
+                continue;
+            }
+            (*replaceImage)->scanLine(x + left, y + top, fillColor);
+        }
+    }
+
+    delete[] filledPixels;
+
+    return true;
+}
+
+// Flood filling based on this scanline algorithm
+// ----- http://lodev.org/cgtutor/floodfill.html
+bool* BitmapImage::floodFillPoints(const BitmapImage* targetImage,
+                                   QRect searchBounds,
+                                   const QRect& maxBounds,
+                                   QPoint point,
+                                   const int tolerance,
+                                   QRect& newBounds,
+                                   bool& fillBorder)
+{
+    QRgb oldColor = targetImage->constScanLine(point.x(), point.y());
     oldColor = qRgba(qRed(oldColor), qGreen(oldColor), qBlue(oldColor), qAlpha(oldColor));
+    QRect borderBounds = searchBounds.intersected(maxBounds);
+    searchBounds = searchBounds.adjusted(-1, -1, 1, 1).intersected(maxBounds);
 
     // Preparations
     QList<QPoint> queue; // queue all the pixels of the filled area (as they are found)
 
-    BitmapImage* replaceImage = nullptr;
     QPoint tempPoint;
-    QRgb newPlacedColor = 0;
     QScopedPointer< QHash<QRgb, bool> > cache(new QHash<QRgb, bool>());
 
     int xTemp = 0;
     bool spanLeft = false;
     bool spanRight = false;
 
-    // Extend to size of Camera
-    targetImage->extend(cameraRect);
-    replaceImage = new BitmapImage(targetImage->mBounds, Qt::transparent);
+    if (!searchBounds.contains(point))
+    {
+        // If point is outside the search area, move it anywhere in the 1px transparent border
+        point = searchBounds.topLeft();
+    }
 
     queue.append(point);
     // Preparations END
 
+    bool *filledPixels = new bool[maxBounds.height()*maxBounds.width()]{};
+
+    // True if the algorithm has attempted to fill a pixel outside the search bounds
+    bool checkOutside = false;
+
+    BlitRect blitBounds(point);
     while (!queue.empty())
     {
         tempPoint = queue.takeFirst();
@@ -882,47 +917,151 @@ void BitmapImage::floodFill(BitmapImage* targetImage,
 
         xTemp = point.x();
 
-        newPlacedColor = replaceImage->constScanLine(xTemp, point.y());
-        while (xTemp >= targetImage->mBounds.left() &&
+        int xCoord = xTemp - maxBounds.left();
+        int yCoord = point.y() - maxBounds.top();
+
+        // In case we fill outside the searchBounds, expand the search area to the max.
+        if (!borderBounds.contains(point)) {
+            checkOutside = true;
+        }
+
+        if (filledPixels[yCoord*maxBounds.width()+xCoord]) continue;
+
+        while (xTemp >= searchBounds.left() &&
                compareColor(targetImage->constScanLine(xTemp, point.y()), oldColor, tolerance, cache.data())) xTemp--;
         xTemp++;
 
         spanLeft = spanRight = false;
-        while (xTemp <= targetImage->mBounds.right() &&
-               compareColor(targetImage->constScanLine(xTemp, point.y()), oldColor, tolerance, cache.data()) &&
-               newPlacedColor != newColor)
+        while (xTemp <= searchBounds.right() &&
+               compareColor(targetImage->constScanLine(xTemp, point.y()), oldColor, tolerance, cache.data()))
         {
 
-            // Set pixel color
-            replaceImage->scanLine(xTemp, point.y(), newColor);
+            QPoint floodPoint = QPoint(xTemp, point.y());
+            if (!blitBounds.contains(floodPoint)) {
+                blitBounds.extend(floodPoint);
+            }
 
-            if (!spanLeft && (point.y() > targetImage->mBounds.top()) &&
+            xCoord = xTemp - maxBounds.left();
+            // This pixel is what we're going to fill later
+            filledPixels[yCoord*maxBounds.width()+xCoord] = true;
+
+            if (!spanLeft && (point.y() > searchBounds.top()) &&
                 compareColor(targetImage->constScanLine(xTemp, point.y() - 1), oldColor, tolerance, cache.data())) {
                 queue.append(QPoint(xTemp, point.y() - 1));
                 spanLeft = true;
             }
-            else if (spanLeft && (point.y() > targetImage->mBounds.top()) &&
+            else if (spanLeft && (point.y() > searchBounds.top()) &&
                      !compareColor(targetImage->constScanLine(xTemp, point.y() - 1), oldColor, tolerance, cache.data())) {
                 spanLeft = false;
             }
 
-            if (!spanRight && point.y() < targetImage->mBounds.bottom() &&
+            if (!spanRight && point.y() < searchBounds.bottom() &&
                 compareColor(targetImage->constScanLine(xTemp, point.y() + 1), oldColor, tolerance, cache.data())) {
                 queue.append(QPoint(xTemp, point.y() + 1));
                 spanRight = true;
-
             }
-            else if (spanRight && point.y() < targetImage->mBounds.bottom() &&
+            else if (spanRight && point.y() < searchBounds.bottom() &&
                      !compareColor(targetImage->constScanLine(xTemp, point.y() + 1), oldColor, tolerance, cache.data())) {
                 spanRight = false;
             }
 
-            Q_ASSERT(queue.count() < (targetImage->mBounds.width() * targetImage->mBounds.height()));
+            Q_ASSERT(queue.count() < (maxBounds.width() * maxBounds.height()));
             xTemp++;
         }
     }
 
-    targetImage->paste(replaceImage);
-    targetImage->modification();
-    delete replaceImage;
+    fillBorder = checkOutside && compareColor(qRgba(0,0,0,0), oldColor, tolerance, cache.data());
+
+    newBounds = blitBounds;
+
+    return filledPixels;
 }
+
+/** Finds all pixels closest to the input color and applies the input color to the image
+ *
+ * An example:
+ *
+ * 0 is where the color was found
+ * 1 is the distance from the nearest pixel of that color
+ *
+ * 211112
+ * 100001
+ * 100001
+ * 211112
+ *
+ * @param bitmapImage: Image to search
+ * @param searchColor: Color to find
+ */
+void BitmapImage::expandFill(bool* fillPixels, const QRect& searchBounds, const QRect& maxBounds, int expand) {
+
+    const int maxWidth = maxBounds.width();
+    const int length = maxBounds.height() * maxBounds.width();
+
+    int* manhattanPoints = new int[length]{};
+
+    // Fill points with max length, this is important because otherwise the filled pixels will include a border of the expanded area
+    std::fill_n(manhattanPoints, length, searchBounds.width()+searchBounds.height());
+
+    for (int y = searchBounds.top(); y <= searchBounds.bottom(); y++)
+    {
+        for (int x = searchBounds.left(); x <= searchBounds.right(); x++)
+        {
+            const int index = y*maxWidth+x;
+
+            if (fillPixels[index]) {
+                manhattanPoints[index] = 0;
+                continue;
+            }
+
+            if (y > searchBounds.top()) {
+                // the value will be the num of pixels away from y - 1 of the next position
+                manhattanPoints[index] = qMin(manhattanPoints[index],
+                                             manhattanPoints[(y - 1) * maxWidth+x] + 1);
+
+                int distance = manhattanPoints[index];
+                if (distance <= expand) {
+                    fillPixels[index] = true;
+                }
+            }
+            if (x > searchBounds.left()) {
+                // the value will be the num of pixels away from x - 1 of the next position
+                manhattanPoints[index] = qMin(manhattanPoints[index],
+                                             manhattanPoints[y*maxWidth+(x - 1)] + 1);
+
+                int distance = manhattanPoints[index];
+                if (distance <= expand) {
+                    fillPixels[index] = true;
+                }
+            }
+        }
+    }
+
+    // traverse from bottom right to top left
+    for (int y = searchBounds.bottom(); y >= searchBounds.top(); y--)
+    {
+        for (int x = searchBounds.right(); x >= searchBounds.left(); x--)
+        {
+            const int index = y*maxWidth+x;
+
+            if (y + 1 < searchBounds.bottom()) {
+                manhattanPoints[index] = qMin(manhattanPoints[index], manhattanPoints[(y + 1)*maxWidth+x] + 1);
+
+                int distance = manhattanPoints[index];
+                if (distance <= expand) {
+                    fillPixels[index] = true;
+                }
+            }
+            if (x + 1 < searchBounds.right()) {
+                manhattanPoints[index] = qMin(manhattanPoints[index], manhattanPoints[y*maxWidth+(x + 1)] + 1);
+
+                int distance = manhattanPoints[index];
+                if (distance <= expand) {
+                    fillPixels[index] = true;
+                }
+            }
+        }
+    }
+
+    delete[] manhattanPoints;
+}
+
