@@ -34,7 +34,7 @@ BezierCurve::BezierCurve()
 BezierCurve::BezierCurve(const QList<QPointF>& pointList, bool smooth)
 {
     QList<qreal> pressureList;
-	for (int i = 0; i < pointList.size(); i++)
+    for (int i = 0; i < pointList.size(); i++)
     {
         pressureList << 0.5; // default pressure
     }
@@ -279,7 +279,7 @@ void BezierCurve::setFilled(bool YesOrNo)
     mFilled = YesOrNo;
 }
 
-BezierCurve BezierCurve::transformed(QTransform transformation)
+BezierCurve BezierCurve::transformed(QTransform transformation) const
 {
     BezierCurve transformedCurve = *this; // copy the curve
     if (isSelected(-1)) { transformedCurve.setOrigin(transformation.map(origin)); }
@@ -297,13 +297,13 @@ BezierCurve BezierCurve::transformed(QTransform transformation)
     if (isSelected(-1)) { newOrigin =  transformation.map(newOrigin); }
     transformedCurve.setOrigin( newOrigin );
     for(int i=0; i< vertex.size(); i++) {
-    	QPointF newC1 = c1.at(i);
-    	QPointF newC2 = c2.at(i);
-    	QPointF newVertex = vertex.at(i);
-    	if (isSelected(i-1)) { newC1 = transformation.map(newC1); }
-    	if (isSelected(i)) { newC2 = transformation.map(newC2);  newVertex = transformation.map(newVertex); }
-    	transformedCurve.appendCubic( newC1, newC2, newVertex, pressure.at(i) );
-    	if (isSelected(i)) { transformedCurve.setSelected(i, true); }
+        QPointF newC1 = c1.at(i);
+        QPointF newC2 = c2.at(i);
+        QPointF newVertex = vertex.at(i);
+        if (isSelected(i-1)) { newC1 = transformation.map(newC1); }
+        if (isSelected(i)) { newC2 = transformation.map(newC2);  newVertex = transformation.map(newVertex); }
+        transformedCurve.appendCubic( newC1, newC2, newVertex, pressure.at(i) );
+        if (isSelected(i)) { transformedCurve.setSelected(i, true); }
     }
     transformedCurve.setWidth( width);
     transformedCurve.setVariableWidth( variableWidth );
@@ -429,9 +429,9 @@ void BezierCurve::removeVertex(int i)
     }
 }
 
-void BezierCurve::drawPath(QPainter& painter, Object* object, QTransform transformation, bool simplified, bool showThinLines )
+void BezierCurve::drawPath(QPainter& painter, const Object& object, QTransform transformation, bool simplified, bool showThinLines )
 {
-    QColor color = object->getColor(colorNumber).color;
+    QColor color = object.getColor(colorNumber).color;
 
     BezierCurve myCurve;
     if (isPartlySelected()) { myCurve = (transformed(transformation)); }
@@ -621,7 +621,8 @@ QPainterPath BezierCurve::getStrokedPath(qreal width, bool usePressure)
 
 QRectF BezierCurve::getBoundingRect()
 {
-    return getSimplePath().boundingRect();
+    qreal radius = getWidth() / 2;
+    return getSimplePath().boundingRect().adjusted(-radius, -radius, radius, radius);
 }
 
 void BezierCurve::createCurve(const QList<QPointF>& pointList, const QList<qreal>& pressureList, bool smooth)
@@ -859,12 +860,16 @@ bool BezierCurve::findIntersection(BezierCurve curve1, int i1, BezierCurve curve
 
     QPointF intersectionPoint = QPointF(50.0, 50.0); // bogus point
     QPointF* cubicIntersection = &intersectionPoint;
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    if ( R1.intersects(R2) || L2.intersects(L1, cubicIntersection) == QLineF::BoundedIntersection )
+#else
     if ( R1.intersects(R2) || L2.intersect(L1, cubicIntersection) == QLineF::BoundedIntersection )
+#endif
     {
         //if (L2.intersect(L1, intersection) == QLineF::BoundedIntersection) {
         //qDebug() << "                   FOUND rectangle intersection ";
         //if (intersectionPoint != curve1.getVertex(i1-1) && intersectionPoint != curve1.getVertex(i1)) {
-        //	qDebug() << "                   it's not one of the points ";
+        //    qDebug() << "                   it's not one of the points ";
         // find the cubic intersection
         int nSteps = 24;
         P1 = curve1.getVertex(i1-1);
@@ -879,7 +884,11 @@ bool BezierCurve::findIntersection(BezierCurve curve1, int i1, BezierCurve curve
                 Q2 = curve2.getPointOnCubic(i2, t);
                 L1 = QLineF(P1, Q1);
                 L2 = QLineF(P2, Q2);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+                if (L2.intersects(L1, cubicIntersection) == QLineF::BoundedIntersection)
+#else
                 if (L2.intersect(L1, cubicIntersection) == QLineF::BoundedIntersection)
+#endif
                 {
                     QPointF intersectionPoint = *cubicIntersection;
                     if (intersectionPoint != curve1.getVertex(i1-1) && intersectionPoint != curve1.getVertex(i1))
