@@ -74,7 +74,7 @@ public:
     void setSelected(int i, bool YesOrNo);
     void setFilled(bool yesOrNo);
 
-    BezierCurve transformed(QTransform transformation);
+    BezierCurve transformed(QTransform transformation) const;
     void transform(QTransform transformation);
 
     void appendCubic(const QPointF& c1Point, const QPointF& c2Point, const QPointF& vertexPoint, qreal pressureValue);
@@ -89,7 +89,7 @@ public:
     QPainterPath getStrokedPath(qreal width, bool pressure);
     QRectF getBoundingRect();
 
-    void drawPath(QPainter& painter, Object* object, QTransform transformation, bool simplified, bool showThinLines );
+    void drawPath(QPainter& painter, const Object& object, QTransform transformation, bool simplified, bool showThinLines );
     void createCurve(const QList<QPointF>& pointList, const QList<qreal>& pressureList , bool smooth);
     void smoothCurve();
 
