@@ -33,9 +33,9 @@ VectorImage::VectorImage()
 VectorImage::VectorImage(const VectorImage& v2) : KeyFrame(v2)
 {
     deselectAll();
-    mObject = v2.mObject;
     mCurves = v2.mCurves;
     mArea = v2.mArea;
+    mOpacity = v2.mOpacity;
 }
 
 VectorImage::~VectorImage()
@@ -51,16 +51,18 @@ VectorImage& VectorImage::operator=(const VectorImage& a) {
 
     deselectAll();
     KeyFrame::operator=(a);
-    mObject = a.mObject;
     mCurves = a.mCurves;
     mArea = a.mArea;
+    mOpacity = a.mOpacity;
     modification();
     return *this;
 }
 
-VectorImage* VectorImage::clone()
+VectorImage* VectorImage::clone() const
 {
-    return new VectorImage(*this);
+    VectorImage* v = new VectorImage(*this);
+    v->setFileName(""); // don't link to the file of the source vector image
+    return v;
 }
 
 /**
@@ -410,7 +412,7 @@ void VectorImage::checkCurveExtremity(BezierCurve& newCurve, qreal tolerance)
                     if (dist1 + dist2 <= 3 * tol3)   // pre-selection, to speed up
                     {
                         QPointF nearestPoint = Q;
-                        qreal t = -1.0;;
+                        qreal t = -1.0;
                         qreal distance = BezierCurve::findDistance(mCurves[i], j, Q, nearestPoint, t);
                         if (distance < tolerance)
                         {
@@ -519,7 +521,7 @@ void VectorImage::checkCurveIntersections(BezierCurve& newCurve, qreal tolerance
                     {
                         // TO DO: find a better intersection point
                         QPointF nearestPoint = Q;
-                        qreal t = -1.0;;
+                        qreal t = -1.0;
                         qreal distance = BezierCurve::findDistance(newCurve, k, Q, nearestPoint, t);
                         //qDebug() << "OK2" << t;
                         if (distance < tolerance)
@@ -859,6 +861,21 @@ void VectorImage::setSelectionRect(QRectF rectangle)
     select(rectangle);
 }
 
+QRectF VectorImage::getBoundsOfTransformedCurves() const
+{
+    QRectF bounds;
+    for (int i = 0; i < mCurves.size(); i++)
+    {
+        BezierCurve curve;
+        if (mCurves.at(i).isPartlySelected())
+        {
+            curve = mCurves[i].transformed(mSelectionTransformation);
+            bounds |= curve.getBoundingRect();
+        }
+    }
+    return bounds;
+}
+
 /**
  * @brief VectorImage::calculateSelectionRect
  */
@@ -1071,16 +1088,6 @@ void VectorImage::paste(VectorImage& vectorImage)
     modification();
 }
 
-/**
- * @brief VectorImage::getColor
- * @param colorNumber: the color number which is referred to in the palette
- * @return QColor
- */
-QColor VectorImage::getColor(int colorNumber)
-{
-    return mObject->getColor(colorNumber).color;
-}
-
 /**
  * @brief VectorImage::getColorNumber
  * @param point: The QPoint of the BezierArea
@@ -1146,11 +1153,17 @@ void VectorImage::removeColor(int index)
 {
     for (int i = 0; i < mArea.size(); i++)
     {
-        if (mArea[i].getColorNumber() > index) mArea[i].decreaseColorNumber();
+        int colorNumber = mArea[i].getColorNumber();
+        if (colorNumber >= index && colorNumber > 0) {
+            mArea[i].decreaseColorNumber();
+        }
     }
     for (int i = 0; i < mCurves.size(); i++)
     {
-        if (mCurves[i].getColorNumber() > index) mCurves[i].decreaseColorNumber();
+        int colorNumber = mCurves[i].getColorNumber();
+        if (colorNumber >= index && colorNumber > 0) {
+            mCurves[i].decreaseColorNumber();
+        }
     }
 }
 
@@ -1169,19 +1182,21 @@ void VectorImage::moveColor(int start, int end)
 /**
  * @brief VectorImage::paintImage
  * @param painter: QPainter&
+ * @param object: const Object&
  * @param simplified: bool
  * @param showThinCurves: bool
  * @param antialiasing: bool
  */
 void VectorImage::paintImage(QPainter& painter,
+    const Object& object,
     bool simplified,
     bool showThinCurves,
     bool antialiasing)
 {
+    painter.save();
     painter.setRenderHint(QPainter::Antialiasing, antialiasing);
 
     painter.setClipping(false);
-    painter.setOpacity(1.0);
     QTransform painterMatrix = painter.transform();
 
     QRect mappedViewRect = QRect(0, 0, painter.device()->width(), painter.device()->height());
@@ -1195,7 +1210,7 @@ void VectorImage::paintImage(QPainter& painter,
             updateArea(mArea[i]); // to do: if selected
 
             // --- fill areas ---- //
-            QColor color = getColor(mArea[i].mColorNumber);
+            QColor color = object.getColor(mArea[i].mColorNumber).color;
 
             painter.save();
             painter.setWorldMatrixEnabled(false);
@@ -1221,29 +1236,10 @@ void VectorImage::paintImage(QPainter& painter,
     // ---- draw curves ----
     for (BezierCurve curve : mCurves)
     {
-        curve.drawPath(painter, mObject, mSelectionTransformation, simplified, showThinCurves);
+        curve.drawPath(painter, object, mSelectionTransformation, simplified, showThinCurves);
         painter.setClipping(false);
     }
-}
-
-/**
- * @brief VectorImage::outputImage
- * @param image: QImage*
- * @param myView: QTransform
- * @param simplified: bool
- * @param showThinCurves: bool
- * @param antialiasing: bool
- */
-void VectorImage::outputImage(QImage* image,
-                              QTransform myView,
-                              bool simplified,
-                              bool showThinCurves,
-                              bool antialiasing)
-{
-    image->fill(qRgba(0, 0, 0, 0));
-    QPainter painter(image);
-    painter.setTransform(myView);
-    paintImage(painter, simplified, showThinCurves, antialiasing);
+    painter.restore();
 }
 
 /**
