@@ -18,17 +18,31 @@ GNU General Public License for more details.
 #ifndef EDITOR_H
 #define EDITOR_H
 
+#include <functional>
 #include <memory>
 #include <QObject>
 #include "pencilerror.h"
 #include "pencildef.h"
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+Q_MOC_INCLUDE("colormanager.h")
+Q_MOC_INCLUDE("toolmanager.h")
+Q_MOC_INCLUDE("layermanager.h")
+Q_MOC_INCLUDE("playbackmanager.h")
+Q_MOC_INCLUDE("viewmanager.h")
+Q_MOC_INCLUDE("preferencemanager.h")
+Q_MOC_INCLUDE("selectionmanager.h")
+Q_MOC_INCLUDE("soundmanager.h")
+Q_MOC_INCLUDE("overlaymanager.h")
+Q_MOC_INCLUDE("clipboardmanager.h")
+#endif
 
-class QDragEnterEvent;
-class QDropEvent;
+class QClipboard;
 class QTemporaryDir;
 class Object;
 class KeyFrame;
+class BitmapImage;
+class VectorImage;
 class LayerCamera;
 class MainWindow2;
 class BaseManager;
@@ -40,10 +54,13 @@ class ViewManager;
 class PreferenceManager;
 class SelectionManager;
 class SoundManager;
+class OverlayManager;
+class ClipboardManager;
 class ScribbleArea;
 class TimeLine;
 class BackupElement;
 class ActiveFramePool;
+class Layer;
 
 enum class SETTING;
 
@@ -58,7 +75,9 @@ class Editor : public QObject
         Q_PROPERTY(ViewManager*     view     READ view)
         Q_PROPERTY(PreferenceManager* preference READ preference)
         Q_PROPERTY(SoundManager*    sound    READ sound)
-        Q_PROPERTY(SelectionManager* select READ select)
+        Q_PROPERTY(SelectionManager* select  READ select)
+        Q_PROPERTY(OverlayManager*  overlays READ overlays)
+        Q_PROPERTY(ClipboardManager* clipboards READ clipboards)
 
 public:
     explicit Editor(QObject* parent = nullptr);
@@ -77,8 +96,11 @@ public:
     PreferenceManager* preference() const { return mPreferenceManager; }
     SoundManager*      sound() const { return mSoundManager; }
     SelectionManager*  select() const { return mSelectionManager; }
+    OverlayManager*    overlays() const { return mOverlayManager; }
+    ClipboardManager*  clipboards() const { return mClipboardManager; }
 
     Object* object() const { return mObject.get(); }
+    Status openObject(const QString& strFilePath, const std::function<void(int)>& progressChanged, const std::function<void(int)>& progressRangeChanged);
     Status setObject(Object* object);
     void updateObject();
     void prepareSave();
@@ -86,7 +108,7 @@ public:
     void setScribbleArea(ScribbleArea* pScirbbleArea) { mScribbleArea = pScirbbleArea; }
     ScribbleArea* getScribbleArea() { return mScribbleArea; }
 
-    int currentFrame();
+    int currentFrame() const;
     int fps();
     void setFps(int fps);
 
@@ -102,8 +124,10 @@ public:
     LayerVisibility layerVisibility();
 
     qreal viewScaleInversed();
-    void deselectAll();
-    void selectAll();
+    void deselectAll() const;
+    void selectAll() const;
+
+    void clipboardChanged();
 
     // backup
     int mBackupIndex;
@@ -118,44 +142,39 @@ signals:
     /** This should be emitted after modifying the frame content */
     void frameModified(int frameNumber);
 
-    /** This should be emitted after the object has been changed */
-    void objectChanged();
-
     /** This should be emitted after modifying multiple frames */
     void framesModified();
+    void selectedFramesChanged();
 
-    void updateTimeLine();
+    void updateTimeLine() const;
+    void updateTimeLineCached();
     void updateLayerCount();
     void updateBackup();
 
     void objectLoaded();
 
-    void changeThinLinesButton(bool);
     void fpsChanged(int fps);
 
     void needSave();
     void needDisplayInfo(const QString& title, const QString& body);
     void needDisplayInfoNoTitle(const QString& body);
 
+    void canCopyChanged(bool enabled);
+    void canPasteChanged(bool enabled);
+
 public: //slots
 
     /** Will call update() and update the canvas
      * Only call this directly If you need the cache to be intact and require the frame to be repainted
-     * Convenient method that does the same as updateFrame but for the current frame
     */
-    void updateCurrentFrame();
+    void updateFrame();
 
-    /** Will call update() and update the canvas
-     * Only call this directly If you need the cache to be intact and require the frame to be repainted
-    */
-    void updateFrame(int frameNumber);
+    void setModified(int layerNumber, int frameNumber);
 
     void clearCurrentFrame();
 
-    void cut();
-
-    bool importImage(QString filePath);
-    bool importGIF(QString filePath, int numOfImages = 0);
+    Status importImage(const QString& filePath);
+    Status importAnimatedImage(const QString& filePath, int frameSpacing, const std::function<void (int)>& progressChanged, const std::function<bool ()>& wasCanceled);
     void restoreKey();
 
     void scrubNextKeyFrame();
@@ -163,16 +182,21 @@ public: //slots
     void scrubForward();
     void scrubBackward();
 
+    /**
+     * Attempts to create a new keyframe at the current frame and layer. If the current frame already holds a keyframe,
+     * the new one will instead be created on the first empty frame that follows. If the current layer is not visible, a
+     * warning dialog will be shown to the user and no new keyframe is created.
+     * @return The newly created keyframe or `nullptr` if the layer is not visible
+     */
     KeyFrame* addNewKey();
     void removeKey();
 
-    void notifyAnimationLengthChanged();
     void switchVisibilityOfLayer(int layerNumber);
     void swapLayers(int i, int j);
-    Status pegBarAlignment(QStringList layers);
+    bool canSwapLayers(int layerIndexLeft, int layerIndexRight) const;
 
-    void backup(QString undoText);
-    void backup(int layerNumber, int frameNumber, QString undoText);
+    void backup(const QString& undoText);
+    bool backup(int layerNumber, int frameNumber, const QString& undoText);
     /**
      * Restores integrity of the backup elements after a layer has been deleted.
      * Removes backup elements affecting the deleted layer and adjusts the layer
@@ -184,17 +208,23 @@ public: //slots
      *          (see #1412).
      */
     void sanitizeBackupElementsAfterLayerDeletion(int layerIndex);
+
+    void onCurrentLayerWillChange(int index);
     void undo();
     void redo();
-    void copy();
 
+    void copy();
+    void copyAndCut();
+    void pasteFromPreviousFrame();
     void paste();
-    void clipboardChanged();
+
+    bool canCopy() const;
+    bool canPaste() const;
+
     void increaseLayerVisibilityIndex();
     void decreaseLayerVisibilityIndex();
     void flipSelection(bool flipVertical);
-
-    void toggleOnionSkinType();
+    void repositionImage(QPoint transform, int frame);
 
     void clearTemporary();
     void addTemporaryDir(QTemporaryDir* dir);
@@ -205,19 +235,17 @@ public: //slots
     bool autoSaveNeverAskAgain() const { return mAutosaveNeverAskAgain; }
     void resetAutoSaveCounter();
 
-    void createNewBitmapLayer(const QString& name);
-    void createNewVectorLayer(const QString& name);
-    void createNewSoundLayer(const QString& name);
-    void createNewCameraLayer(const QString& name);
-
-protected:
-    // Need to move to somewhere...
-    void dragEnterEvent(QDragEnterEvent*);
-    void dropEvent(QDropEvent*);
-
 private:
-    bool importBitmapImage(QString, int space = 0);
-    bool importVectorImage(QString);
+    Status importBitmapImage(const QString&);
+    Status importVectorImage(const QString&);
+
+    void pasteToCanvas(BitmapImage* bitmapImage, int frameNumber);
+    void pasteToCanvas(VectorImage* vectorImage, int frameNumber);
+    void pasteToFrames();
+
+    bool canCopyBitmapImage(BitmapImage* bitmapImage) const;
+    bool canCopyFrames(const Layer* layer) const;
+    bool canCopyVectorImage(const VectorImage* vectorImage) const;
 
     // the object to be edited by the editor
     std::unique_ptr<Object> mObject;
@@ -234,7 +262,9 @@ private:
     ViewManager*       mViewManager = nullptr;
     PreferenceManager* mPreferenceManager = nullptr;
     SoundManager*      mSoundManager = nullptr;
-    SelectionManager* mSelectionManager = nullptr;
+    SelectionManager*  mSelectionManager = nullptr;
+    OverlayManager*    mOverlayManager = nullptr;
+    ClipboardManager*  mClipboardManager = nullptr;
 
     std::vector< BaseManager* > mAllManagers;
 
@@ -244,7 +274,16 @@ private:
     bool mAutosaveNeverAskAgain = false;
 
     void makeConnections();
-    KeyFrame* addKeyFrame(int layerNumber, int frameNumber);
+    /**
+     * Attempts to create a new keyframe at the given position and layer. If a keyframe already exists at the position,
+     * the new one will instead be created on the first empty frame that follows. If the layer is not visible, a warning
+     * dialog will be shown to the user and no new keyframe is created.
+     * @param layerNumber The number of an existing layer on which the keyframe is to be created
+     * @param frameIndex The desired position of the new keyframe
+     * @return The newly created keyframe or `nullptr` if the layer is not visible
+     * @see Editor::addNewKey()
+     */
+    KeyFrame* addKeyFrame(int layerNumber, int frameIndex);
 
     QList<QTemporaryDir*> mTemporaryDirs;
 
@@ -253,11 +292,6 @@ private:
     void updateAutoSaveCounter();
     int mLastModifiedFrame = -1;
     int mLastModifiedLayer = -1;
-
-    // clipboard
-    bool clipboardBitmapOk = true;
-    bool clipboardVectorOk = true;
-    bool clipboardSoundClipOk = true;
 };
 
 #endif
