@@ -34,9 +34,11 @@ GNU General Public License for more details.
 #include "pencildef.h"
 #include "bitmapimage.h"
 #include "canvaspainter.h"
+#include "overlaypainter.h"
 #include "preferencemanager.h"
-#include "strokemanager.h"
 #include "selectionpainter.h"
+#include "camerapainter.h"
+#include "tiledbuffer.h"
 
 class Layer;
 class Editor;
@@ -51,7 +53,6 @@ class ScribbleArea : public QWidget
     Q_OBJECT
 
     friend class MoveTool;
-    friend class EditTool;
     friend class SmudgeTool;
     friend class BucketTool;
 
@@ -61,37 +62,24 @@ public:
 
     bool init();
     void setEditor(Editor* e) { mEditor = e; }
-    StrokeManager* getStrokeManager() const { return mStrokeManager.get(); }
     Editor* editor() const { return mEditor; }
 
     void deleteSelection();
-    void applySelectionChanges();
-    void displaySelectionProperties();
 
-    void paintTransformedSelection();
     void applyTransformedSelection();
     void cancelTransformedSelection();
 
-    bool isLayerPaintable() const;
-
-    QVector<QPoint> calcSelectionCenterPoints();
-
     void setEffect(SETTING e, bool isOn);
 
     LayerVisibility getLayerVisibility() const { return mLayerVisibility; }
     qreal getCurveSmoothing() const { return mCurveSmoothingLevel; }
-    bool usePressure() const { return mUsePressure; }
     bool makeInvisible() const { return mMakeInvisible; }
 
-    QRectF getCameraRect();
     QPointF getCentralPoint();
 
-    /** Update current frame.
-     *  calls update() behind the scene and update cache if necessary */
-    void updateCurrentFrame();
     /** Update frame.
      * calls update() behind the scene and update cache if necessary */
-    void updateFrame(int frame);
+    void updateFrame();
 
     /** Frame scrubbed, invalidate relevant cache */
     void onScrubbed(int frameNumber);
@@ -108,10 +96,6 @@ public:
     /** Frame modified, invalidate cache for frame if any */
     void onFrameModified(int frameNumber);
 
-    /** Current frame modified, invalidate current frame cache if any.
-     * Convenient function that does the same as onFrameModified */
-    void onCurrentFrameModified();
-
     /** Layer changed, invalidate relevant cache */
     void onLayerChanged();
 
@@ -123,38 +107,35 @@ public:
     void onOnionSkinTypeChanged();
 
     /** Object updated, invalidate all cache */
-    void onObjectChanged();
+    void onObjectLoaded();
 
-    /** Set frame on layer to modified and invalidate current frame cache */
-    void setModified(int layerNumber, int frameNumber);
+    /** Tool property updated, invalidate cache and frame if needed */
+    void onToolPropertyUpdated(ToolType, ToolPropertyType);
+
+    /** Tool changed, invalidate cache and frame if needed */
+    void onToolChanged(ToolType);
+
+    void endStroke();
 
     void flipSelection(bool flipVertical);
 
     BaseTool* currentTool() const;
-    BaseTool* getTool(ToolType eToolMode);
-    void setCurrentTool(ToolType eToolMode);
-    void setTemporaryTool(ToolType eToolMode);
-    void setPrevTool();
-
-    void floodFillError(int errorType);
 
     bool isMouseInUse() const { return mMouseInUse; }
     bool isTabletInUse() const { return mTabletInUse; }
     bool isPointerInUse() const { return mMouseInUse || mTabletInUse; }
-    bool isTemporaryTool() const { return mInstantTool; }
 
     void keyEvent(QKeyEvent* event);
     void keyEventForSelection(QKeyEvent* event);
 
 signals:
     void multiLayerOnionSkinChanged(bool);
-    void refreshPreview();
+    void selectionUpdated();
 
 public slots:
     void clearImage();
     void setCurveSmoothing(int);
     void toggleThinLines();
-    void toggleOutlines();
     void increaseLayerVisibilityIndex();
     void decreaseLayerVisibilityIndex();
     void setLayerVisibility(LayerVisibility visibility);
@@ -163,7 +144,8 @@ public slots:
     void paletteColorChanged(QColor);
 
     void showLayerNotVisibleWarning();
-
+    void onTileUpdated(TiledBuffer* tiledBuffer, Tile* tile);
+    void onTileCreated(TiledBuffer* tiledBuffer, Tile* tile);
 
 protected:
     bool event(QEvent *event) override;
@@ -180,44 +162,33 @@ protected:
 
 public:
     void drawPolyline(QPainterPath path, QPen pen, bool useAA);
-    void drawLine(QPointF P1, QPointF P2, QPen pen, QPainter::CompositionMode cm);
     void drawPath(QPainterPath path, QPen pen, QBrush brush, QPainter::CompositionMode cm);
     void drawPen(QPointF thePoint, qreal brushWidth, QColor fillColor, bool useAA = true);
     void drawPencil(QPointF thePoint, qreal brushWidth, qreal fixedBrushFeather, QColor fillColor, qreal opacity);
-    void drawBrush(QPointF thePoint, qreal brushWidth, qreal offset, QColor fillColor, qreal opacity, bool usingFeather = true, bool useAA = false);
+    void drawBrush(QPointF thePoint, qreal brushWidth, qreal offset, QColor fillColor, QPainter::CompositionMode compMode, qreal opacity, bool usingFeather = true, bool useAA = false);
     void blurBrush(BitmapImage *bmiSource_, QPointF srcPoint_, QPointF thePoint_, qreal brushWidth_, qreal offset_, qreal opacity_);
     void liquifyBrush(BitmapImage *bmiSource_, QPointF srcPoint_, QPointF thePoint_, qreal brushWidth_, qreal offset_, qreal opacity_);
 
     void paintBitmapBuffer();
-    void paintBitmapBufferRect(const QRect& rect);
-    void paintCanvasCursor(QPainter& painter);
-    void clearBitmapBuffer();
-    void refreshBitmap(const QRectF& rect, int rad);
-    void refreshVector(const QRectF& rect, int rad);
+    void clearDrawingBuffer();
     void setGaussianGradient(QGradient &gradient, QColor color, qreal opacity, qreal offset);
 
     void pointerPressEvent(PointerEvent*);
     void pointerMoveEvent(PointerEvent*);
     void pointerReleaseEvent(PointerEvent*);
 
-    void updateCanvasCursor();
-
     /// Call this when starting to use a paint tool. Checks whether we are drawing
     /// on an empty frame, and if so, takes action according to use preference.
     void handleDrawingOnEmptyFrame();
 
-    BitmapImage* mBufferImg = nullptr; // used to pre-draw vector modifications
-
-    QPixmap mCursorImg;
-    QPixmap mTransCursImg;
-
+    TiledBuffer mTiledBuffer;
 private:
 
-    /** Invalidate the layer pixmap cache.
+    /** Invalidate the layer pixmap and camera painter caches.
      * Call this in most situations where the layer rendering order is affected.
      * Peviously known as setAllDirty.
     */
-    void invalidateLayerPixmapCache();
+    void invalidatePainterCaches();
 
     /** Invalidate cache for the given frame */
     void invalidateCacheForFrame(int frameNumber);
@@ -232,7 +203,9 @@ private:
     /** invalidate onion skin cache around frame */
     void invalidateOnionSkinsCacheAround(int frame);
 
-    void prepCanvas(int frame, QRect rect);
+    void prepOverlays(int frame);
+    void prepCameraPainter(int frame);
+    void prepCanvas(int frame);
     void drawCanvas(int frame, QRect rect);
     void settingUpdated(SETTING setting);
     void paintSelectionVisuals(QPainter &painter);
@@ -240,33 +213,25 @@ private:
     BitmapImage* currentBitmapImage(Layer* layer) const;
     VectorImage* currentVectorImage(Layer* layer) const;
 
-    MoveMode mMoveMode = MoveMode::NONE;
-    ToolType mPrevTemporalToolType = ERASER;
-    ToolType mPrevToolType = PEN; // previous tool (except temporal)
-
-    BitmapImage mBitmapSelection; // used to temporary store a transformed portion of a bitmap image
-
-    std::unique_ptr<StrokeManager> mStrokeManager;
-
     Editor* mEditor = nullptr;
 
-
-    bool mIsSimplified = false;
-    bool mShowThinLines = false;
-    bool mQuickSizing = true;
     LayerVisibility mLayerVisibility = LayerVisibility::ALL;
-    bool mUsePressure   = true;
     bool mMakeInvisible = false;
-    bool mToolCursors = true;
     qreal mCurveSmoothingLevel = 0.0;
-    bool mMultiLayerOnionSkin = false; // future use. If required, just add a checkbox to updated it.
-    QColor mOnionColor;
+    bool mMultiLayerOnionSkin = false; // Future use. If required, just add a checkbox to update it.
+    int mDeltaFactor = 1;
 
-private:
-    bool mKeyboardInUse = false;
+    /* Under certain circumstances a mouse press event will fire after a tablet release event.
+       This causes unexpected behaviours for some of the tools, eg. the bucket.
+       The problem only seems to occur on windows and only when tapping.
+       prior to this fix the event queue would look like this:
+       eg: TabletPress -> TabletRelease -> MousePress
+       The following will filter mouse events created after a tablet release event.
+    */
+    void tabletReleaseEventFired();
     bool mMouseInUse = false;
-    bool mMouseRightButtonInUse = false;
     bool mTabletInUse = false;
+    qreal mDevicePixelRatio = 1.;
 
     // Double click handling for tablet input
     void handleDoubleClick();
@@ -275,18 +240,21 @@ private:
     // Microsoft suggests that a double click action should be no more than 500 ms
     const int DOUBLE_CLICK_THRESHOLD = 500;
     QTimer* mDoubleClickTimer = nullptr;
+    QPointF mTabletPressPos;
+    int mTabletReleaseMillisAgo;
+    const int MOUSE_FILTER_THRESHOLD = 200;
 
-    QPoint mCursorCenterPos;
-    QPointF mTransformedCursorPos;
-
-    //instant tool (temporal eg. eraser)
-    bool mInstantTool = false; //whether or not using temporal tool
+    QTimer* mMouseFilterTimer = nullptr;
 
     PreferenceManager* mPrefs = nullptr;
 
     QPixmap mCanvas;
     CanvasPainter mCanvasPainter;
+    OverlayPainter mOverlayPainter;
     SelectionPainter mSelectionPainter;
+    CameraPainter mCameraPainter;
+
+    QPolygonF mOriginalPolygonF = QPolygonF();
 
     // Pixmap Cache keys
     QMap<unsigned int, QPixmapCache::Key> mPixmapCacheKeys;
