@@ -42,6 +42,7 @@ public:
     Layer* currentLayer();
     Layer* currentLayer(int offset);
     Layer* getLayer(int index);
+    LayerCamera* getCameraLayerBelow(int layerIndex) const;
     Layer* findLayerByName(QString sName, Layer::LAYER_TYPE type = Layer::UNDEFINED);
     Layer* getLastCameraLayer();
     int    currentLayerIndex();
@@ -49,6 +50,7 @@ public:
     void   setCurrentLayer(Layer* layer);
     int    count();
 
+    bool canDeleteLayer(int index) const;
     Status deleteLayer(int index);
     Status renameLayer(Layer*, const QString& newName);
     void notifyLayerChanged(Layer*);
@@ -56,6 +58,8 @@ public:
     void gotoNextLayer();
     void gotoPreviouslayer();
 
+    /** Returns a new Layer with the given LAYER_TYPE */
+    Layer* createLayer(Layer::LAYER_TYPE type, const QString& strLayerName);
     LayerBitmap* createBitmapLayer(const QString& strLayerName);
     LayerVector* createVectorLayer(const QString& strLayerName);
     LayerCamera* createCameraLayer(const QString& strLayerName);
@@ -75,6 +79,7 @@ public:
     int getLastLayerIndex() { return count() - 1; }
 
 signals:
+    void currentLayerWillChange(int index);
     void currentLayerChanged(int index);
     void layerCountChanged(int count);
     void animationLengthChanged(int length);
