@@ -20,7 +20,7 @@ GNU General Public License for more details.
 #include <QSettings>
 #include <QDebug>
 
-PreferenceManager::PreferenceManager(Editor* editor) : BaseManager(editor)
+PreferenceManager::PreferenceManager(Editor* editor) : BaseManager(editor, __FUNCTION__)
 {
 }
 
@@ -56,6 +56,10 @@ void PreferenceManager::loadPrefs()
     set(SETTING::OVERLAY_THIRDS,           settings.value(SETTING_OVERLAY_THIRDS,         false).toBool());
     set(SETTING::OVERLAY_GOLDEN,           settings.value(SETTING_OVERLAY_GOLDEN,         false).toBool());
     set(SETTING::OVERLAY_SAFE,             settings.value(SETTING_OVERLAY_SAFE,           false).toBool());
+    set(SETTING::OVERLAY_PERSPECTIVE1,     settings.value(SETTING_OVERLAY_PERSPECTIVE1,   false).toBool());
+    set(SETTING::OVERLAY_PERSPECTIVE2,     settings.value(SETTING_OVERLAY_PERSPECTIVE2,   false).toBool());
+    set(SETTING::OVERLAY_PERSPECTIVE3,     settings.value(SETTING_OVERLAY_PERSPECTIVE3,   false).toBool());
+    set(SETTING::OVERLAY_ANGLE,            settings.value(SETTING_OVERLAY_ANGLE,          15).toInt());
     set(SETTING::ACTION_SAFE,              settings.value(SETTING_ACTION_SAFE,            5).toInt());
     set(SETTING::ACTION_SAFE_ON,           settings.value(SETTING_ACTION_SAFE_ON,         true).toBool());
     set(SETTING::OVERLAY_SAFE_HELPER_TEXT_ON, settings.value(SETTING_OVERLAY_SAFE_HELPER_TEXT_ON, true).toBool());
@@ -69,14 +73,19 @@ void PreferenceManager::loadPrefs()
     // General
     set(SETTING::ANTIALIAS,                settings.value(SETTING_ANTIALIAS,              true).toBool());
     set(SETTING::TOOL_CURSOR,              settings.value(SETTING_TOOL_CURSOR,            true).toBool());
-    set(SETTING::DOTTED_CURSOR,            settings.value(SETTING_DOTTED_CURSOR,          true).toBool());
+    set(SETTING::CANVAS_CURSOR,            settings.value(SETTING_CANVAS_CURSOR,          true).toBool());
     set(SETTING::HIGH_RESOLUTION,          settings.value(SETTING_HIGH_RESOLUTION,        true).toBool());
     set(SETTING::SHADOW,                   settings.value(SETTING_SHADOW,                 false).toBool());
     set(SETTING::QUICK_SIZING,             settings.value(SETTING_QUICK_SIZING,           true).toBool());
+    set(SETTING::SHOW_SELECTION_INFO,      settings.value(SETTING_SHOW_SELECTION_INFO,    false).toBool());
+
+    set(SETTING::INVERT_DRAG_ZOOM_DIRECTION,   settings.value(SETTING_INVERT_DRAG_ZOOM_DIRECTION,  false).toBool());
+    set(SETTING::INVERT_SCROLL_ZOOM_DIRECTION, settings.value(SETTING_INVERT_SCROLL_ZOOM_DIRECTION,  false).toBool());
 
     set(SETTING::ROTATION_INCREMENT,       settings.value(SETTING_ROTATION_INCREMENT,     15).toInt());
 
     set(SETTING::WINDOW_OPACITY,           settings.value(SETTING_WINDOW_OPACITY,         0).toInt());
+    set(SETTING::SHOW_STATUS_BAR,          settings.value(SETTING_SHOW_STATUS_BAR,        true).toBool());
     set(SETTING::CURVE_SMOOTHING,          settings.value(SETTING_CURVE_SMOOTHING,        20).toInt());
 
     set(SETTING::BACKGROUND_STYLE,         settings.value(SETTING_BACKGROUND_STYLE,       "white").toString());
@@ -269,6 +278,9 @@ void PreferenceManager::set(SETTING option, int value)
     case SETTING::ONION_NEXT_FRAMES_NUM:
         settings.setValue(SETTING_ONION_NEXT_FRAMES_NUM, value);
         break;
+    case SETTING::OVERLAY_ANGLE:
+        settings.setValue(SETTING_OVERLAY_ANGLE, value);
+        break;
     case SETTING::FLIP_ROLL_MSEC :
         settings.setValue(SETTING_FLIP_ROLL_MSEC, value);
         break;
@@ -343,6 +355,9 @@ void PreferenceManager::set(SETTING option, bool value)
     QSettings settings(PENCIL2D, PENCIL2D);
     switch (option)
     {
+    case SETTING::SHOW_STATUS_BAR:
+        settings.setValue(SETTING_SHOW_STATUS_BAR, value);
+        break;
     case SETTING::ANTIALIAS:
         settings.setValue(SETTING_ANTIALIAS, value);
         break;
@@ -361,6 +376,15 @@ void PreferenceManager::set(SETTING option, bool value)
     case SETTING::OVERLAY_SAFE:
         settings.setValue(SETTING_OVERLAY_SAFE, value);
         break;
+    case SETTING::OVERLAY_PERSPECTIVE1:
+        settings.setValue(SETTING_OVERLAY_PERSPECTIVE1, value);
+        break;
+    case SETTING::OVERLAY_PERSPECTIVE2:
+        settings.setValue(SETTING_OVERLAY_PERSPECTIVE2, value);
+        break;
+    case SETTING::OVERLAY_PERSPECTIVE3:
+        settings.setValue(SETTING_OVERLAY_PERSPECTIVE3, value);
+        break;
     case SETTING::ACTION_SAFE_ON:
         settings.setValue(SETTING_ACTION_SAFE_ON, value);
         break;
@@ -397,8 +421,8 @@ void PreferenceManager::set(SETTING option, bool value)
     case SETTING::TOOL_CURSOR:
         settings.setValue(SETTING_TOOL_CURSOR, value);
         break;
-    case SETTING::DOTTED_CURSOR:
-        settings.setValue(SETTING_DOTTED_CURSOR, value);
+    case SETTING::CANVAS_CURSOR:
+        settings.setValue(SETTING_CANVAS_CURSOR, value);
         break;
     case SETTING::HIGH_RESOLUTION:
         settings.setValue(SETTING_HIGH_RESOLUTION, value);
@@ -415,9 +439,18 @@ void PreferenceManager::set(SETTING option, bool value)
     case SETTING::QUICK_SIZING:
         settings.setValue(SETTING_QUICK_SIZING, value);
         break;
+    case SETTING::INVERT_DRAG_ZOOM_DIRECTION:
+        settings.setValue(SETTING_INVERT_DRAG_ZOOM_DIRECTION, value);
+        break;
+    case SETTING::INVERT_SCROLL_ZOOM_DIRECTION:
+        settings.setValue(SETTING_INVERT_SCROLL_ZOOM_DIRECTION, value);
+        break;
     case SETTING::LAYOUT_LOCK:
         settings.setValue(SETTING_LAYOUT_LOCK, value);
         break;
+    case SETTING::SHOW_SELECTION_INFO:
+        settings.setValue(SETTING_SHOW_SELECTION_INFO, value);
+        break;
     case SETTING::SOUND_SCRUB_ACTIVE:
         settings.setValue(SETTING_SOUND_SCRUB_ACTIVE, value);
         break;
