@@ -19,8 +19,6 @@ GNU General Public License for more details.
 #include "viewmanager.h"
 #include "editor.h"
 #include "object.h"
-#include "camera.h"
-#include "layercamera.h"
 
 const static qreal mMinScale = 0.01;
 const static qreal mMaxScale = 100.0;
@@ -34,14 +32,11 @@ const std::vector<qreal> gZoomLevels
 };
 
 
-ViewManager::ViewManager(Editor* editor) : BaseManager(editor)
+ViewManager::ViewManager(Editor* editor) : BaseManager(editor, __FUNCTION__)
 {
-    mDefaultEditorCamera = new Camera;
-    mCurrentCamera = mDefaultEditorCamera;
 }
 
 ViewManager::~ViewManager() {
-    delete mDefaultEditorCamera;
 }
 
 bool ViewManager::init()
@@ -52,9 +47,6 @@ bool ViewManager::init()
 
 Status ViewManager::load(Object*)
 {
-    mCameraLayer = nullptr;
-    mCurrentCamera = mDefaultEditorCamera;
-    mCurrentCamera->reset();
     updateViewTransforms();
 
     return Status::OK;
@@ -66,18 +58,6 @@ Status ViewManager::save(Object* o)
     return Status::OK;
 }
 
-void ViewManager::workingLayerChanged(Layer* layer)
-{
-    if (layer->type() == Layer::CAMERA)
-    {
-        setCameraLayer(layer);
-    }
-    else
-    {
-        setCameraLayer(nullptr);
-    }
-}
-
 QPointF ViewManager::mapCanvasToScreen(QPointF p) const
 {
     return mViewCanvas.map(p);
@@ -128,26 +108,23 @@ QTransform ViewManager::getViewInverse() const
     return mViewCanvasInverse;
 }
 
+qreal ViewManager::getViewScaleInverse() const
+{
+    return mViewCanvasInverse.m11();
+}
+
 void ViewManager::updateViewTransforms()
 {
-    if (mCameraLayer)
-    {
-        int frame = editor()->currentFrame();
-        mCurrentCamera = mCameraLayer->getCameraAtFrame(frame);
-        if (mCurrentCamera)
-        {
-            mCurrentCamera->updateViewTransform();
-        }
-        mView = mCameraLayer->getViewAtFrame(frame);
-    }
-    else
-    {
-        mCurrentCamera = mDefaultEditorCamera;
-        mCurrentCamera->updateViewTransform();
+    QTransform t;
+    t.translate(mTranslation.x(), mTranslation.y());
 
-        mView = mCurrentCamera->getView();
-    }
+    QTransform r;
+    r.rotate(mRotation);
 
+    QTransform s;
+    s.scale(mScaling, mScaling);
+
+    mView = t * r * s;
     mViewInverse = mView.inverted();
 
     float flipX = mIsFlipHorizontal ? -1.f : 1.f;
@@ -160,22 +137,15 @@ void ViewManager::updateViewTransforms()
 
 QPointF ViewManager::translation() const
 {
-    if (mCurrentCamera)
-    {
-        return mCurrentCamera->translation();
-    }
-    return QPointF(0, 0);
+    return mTranslation;
 }
 
 void ViewManager::translate(float dx, float dy)
 {
-    if (mCurrentCamera)
-    {
-        mCurrentCamera->translate(static_cast<qreal>(dx), static_cast<qreal>(dy));
-        updateViewTransforms();
+    mTranslation = QPointF(dx, dy);
+    updateViewTransforms();
 
-        emit viewChanged();
-    }
+    emit viewChanged();
 }
 
 void ViewManager::translate(QPointF offset)
@@ -190,22 +160,23 @@ void ViewManager::centerView()
 
 float ViewManager::rotation()
 {
-    if (mCurrentCamera)
-    {
-        return static_cast<float>(mCurrentCamera->rotation());
-    }
-    return 0.0f;
+    return mRotation;
 }
 
 void ViewManager::rotate(float degree)
 {
-    if (mCurrentCamera)
-    {
-        mCurrentCamera->rotate(static_cast<qreal>(degree));
-        updateViewTransforms();
+    mRotation = degree;
+    updateViewTransforms();
 
-        emit viewChanged();
-    }
+    emit viewChanged();
+}
+
+void ViewManager::rotateRelative(float delta)
+{
+    mRotation = mRotation + delta;
+    updateViewTransforms();
+
+    emit viewChanged();
 }
 
 void ViewManager::resetRotation()
@@ -215,39 +186,36 @@ void ViewManager::resetRotation()
 
 qreal ViewManager::scaling()
 {
-    if (mCurrentCamera)
-    {
-        return mCurrentCamera->scaling();
-    }
-    return 0.0;
+    return mScaling;
 }
 
 void ViewManager::scaleUp()
 {
     for (size_t i = 0; i < gZoomLevels.size(); i++)
     {
-        if (gZoomLevels[i] > scaling())
+        if (mScaling < gZoomLevels[i])
         {
             scale(gZoomLevels[i]);
             return;
         }
     }
 
-    // out of pre-defined zoom levels
-    scale(scaling() * 1.18);
+    scale(mScaling * 1.25);
 }
 
 void ViewManager::scaleDown()
 {
-    for (int i = static_cast<int>(gZoomLevels.size()) - 1; i >= 0; --i)
+    const size_t nZoomLevels = gZoomLevels.size();
+    for (size_t i = 1; i <= nZoomLevels; i++)
     {
-        if (gZoomLevels[static_cast<unsigned>(i)] < scaling())
+        if (mScaling > gZoomLevels[nZoomLevels - i])
         {
-            scale(gZoomLevels[static_cast<unsigned>(i)]);
+            scale(gZoomLevels[nZoomLevels - i]);
             return;
         }
     }
-    scale(scaling() * 0.8333);
+
+    scale(mScaling * 0.8);
 }
 
 void ViewManager::scale100()
@@ -296,16 +264,13 @@ void ViewManager::scale(qreal scaleValue)
         scaleValue = mMaxScale;
     }
 
-    if (mCurrentCamera)
-    {
-        mCurrentCamera->scale(scaleValue);
-        updateViewTransforms();
+    mScaling = scaleValue;
+    updateViewTransforms();
 
-        emit viewChanged();
-    }
+    emit viewChanged();
 }
 
-void ViewManager::scaleWithOffset(qreal scaleValue, QPointF offset)
+void ViewManager::scaleAtOffset(qreal scaleValue, QPointF offset)
 {
     if (scaleValue < mMinScale)
     {
@@ -315,14 +280,11 @@ void ViewManager::scaleWithOffset(qreal scaleValue, QPointF offset)
     {
         scaleValue = mMaxScale;
     }
+    mTranslation = (mTranslation + offset) * mScaling / scaleValue - offset;
+    mScaling = scaleValue;
+    updateViewTransforms();
 
-    if (mCurrentCamera)
-    {
-        mCurrentCamera->scaleWithOffset(scaleValue, offset);
-        updateViewTransforms();
-
-        emit viewChanged();
-    }
+    emit viewChanged();
 }
 
 void ViewManager::flipHorizontal(bool b)
@@ -398,40 +360,26 @@ void ViewManager::setCanvasSize(QSize size)
     emit viewChanged();
 }
 
-void ViewManager::setCameraLayer(Layer* layer)
+void ViewManager::forceUpdateViewTransform()
 {
-    if (layer != nullptr)
-    {
-        if (layer->type() != Layer::CAMERA)
-        {
-            Q_ASSERT(false && "Only camera layers allowed pls");
-            return;
-        }
-        mCameraLayer = static_cast<LayerCamera*>(layer);
-    }
-    else
-    {
-        mCameraLayer = nullptr;
-    }
-
     updateViewTransforms();
+    emit viewChanged();
 }
 
 void ViewManager::onCurrentFrameChanged()
 {
-    if (mCameraLayer)
-    {
-        updateViewTransforms();
-    }
+    // emit because of potential camera interpolation changes
+    emit viewChanged();
 }
 
 void ViewManager::resetView()
 {
-    if (mCurrentCamera)
-    {
-        mCurrentCamera->reset();
-        updateViewTransforms();
-        emit viewChanged();
-        emit viewFlipped();
-    }
+    mTranslation = QPointF(0,0);
+    mScaling = 1.0;
+    mRotation = 0.0;
+
+    updateViewTransforms();
+    emit viewChanged();
+    emit viewFlipped();
+
 }
