@@ -21,10 +21,6 @@ GNU General Public License for more details.
 #include <QTransform>
 #include "basemanager.h"
 
-class Layer;
-class LayerCamera;
-class Camera;
-
 
 class ViewManager : public BaseManager
 {
@@ -37,10 +33,10 @@ public:
     bool init() override;
     Status load(Object*) override;
     Status save(Object*) override;
-    void workingLayerChanged(Layer* layer) override;
 
     QTransform getView() const;
     QTransform getViewInverse() const;
+    qreal getViewScaleInverse() const;
     void resetView();
 
     QPointF mapCanvasToScreen(QPointF p) const;
@@ -62,11 +58,12 @@ public:
 
     float rotation();
     void rotate(float degree);
+    void rotateRelative(float delta);
     void resetRotation();
 
     qreal scaling();
     void scale(qreal scaleValue);
-    void scaleWithOffset(qreal scaleValue, QPointF offset);
+    void scaleAtOffset(qreal scaleValue, QPointF offset);
     void scaleUp();
     void scaleDown();
     void scale400();
@@ -79,6 +76,7 @@ public:
 
     void flipHorizontal(bool b);
     void flipVertical(bool b);
+
     void setOverlayCenter(bool b);
     void setOverlayThirds(bool b);
     void setOverlayGoldenRatio(bool b);
@@ -91,9 +89,7 @@ public:
     bool getOverlayGoldenRatio() const { return mOverlayGoldenRatio; }
     bool getOverlaySafeAreas() const { return mOverlaySafeAreas; }
 
-
     void setCanvasSize(QSize size);
-    void setCameraLayer(Layer* layer);
 
     QTransform getImportView() { return mImportView; }
     void setImportView(const QTransform& newView) { mImportView = newView; }
@@ -101,14 +97,14 @@ public:
     void setImportFollowsCamera(bool b) { mImportFollowsCamera = b; }
     bool getImportFollowsCamera() { return mImportFollowsCamera; }
 
-    void updateViewTransforms();
+    void forceUpdateViewTransform();
 
 signals:
     void viewChanged();
     void viewFlipped();
 
 private:
-
+    void updateViewTransforms();
     void onCurrentFrameChanged();
 
     QTransform mView;
@@ -118,8 +114,9 @@ private:
     QTransform mCentre;
     QTransform mImportView;
 
-    Camera* mDefaultEditorCamera = nullptr;
-    Camera* mCurrentCamera = nullptr;
+    QPointF mTranslation = QPointF();
+    qreal mScaling = 1.0;
+    qreal mRotation = 0.0;
 
     QSize mCanvasSize = { 1, 1 };
 
@@ -131,8 +128,6 @@ private:
     bool mOverlaySafeAreas = false;
 
     bool mImportFollowsCamera = false;
-
-    LayerCamera* mCameraLayer = nullptr;
 };
 
 #endif // VIEWMANAGER_H
