@@ -27,6 +27,7 @@ GNU General Public License for more details.
 #include <QThread>
 #include <QtMath>
 #include <QPainter>
+#include <QRegularExpression>
 
 #include "object.h"
 #include "layercamera.h"
@@ -34,6 +35,12 @@ GNU General Public License for more details.
 #include "soundclip.h"
 #include "util.h"
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+using Qt::SplitBehaviorFlags;
+#else
+using SplitBehaviorFlags = QString::SplitBehavior;
+#endif
+
 MovieExporter::MovieExporter()
 {
 }
@@ -76,7 +83,7 @@ Status MovieExporter::run(const Object* obj,
 {
     majorProgress(0.f, 0.03f);
     minorProgress(0.f);
-    progressMessage(QObject::tr("Checking environment..."));
+    progressMessage(tr("Checking environment..."));
 
     clock_t t1 = clock();
 
@@ -110,24 +117,24 @@ Status MovieExporter::run(const Object* obj,
     if (desc.strFileName.endsWith("gif", Qt::CaseInsensitive))
     {
         majorProgress(0.03f, 1.f);
-        progressMessage(QObject::tr("Generating GIF..."));
+        progressMessage(tr("Generating GIF..."));
         minorProgress(0.f);
         STATUS_CHECK(generateGif(obj, ffmpegPath, desc.strFileName, minorProgress))
     }
     else
     {
         majorProgress(0.03f, 0.25f);
-        progressMessage(QObject::tr("Assembling audio..."));
+        progressMessage(tr("Assembling audio..."));
         minorProgress(0.f);
         STATUS_CHECK(assembleAudio(obj, ffmpegPath, minorProgress))
         minorProgress(1.f);
         majorProgress(0.25f, 1.f);
-        progressMessage(QObject::tr("Generating movie..."));
+        progressMessage(tr("Generating movie..."));
         STATUS_CHECK(generateMovie(obj, ffmpegPath, desc.strFileName, minorProgress))
     }
     minorProgress(1.f);
     majorProgress(1.f, 1.f);
-    progressMessage(QObject::tr("Done"));
+    progressMessage(tr("Done"));
 
     clock_t t2 = clock() - t1;
     qDebug("MOVIE = %.1f sec", static_cast<double>(t2 / CLOCKS_PER_SEC));
@@ -175,6 +182,7 @@ Status MovieExporter::assembleAudio(const Object* obj,
     std::vector< LayerSound* > allSoundLayers = obj->getLayersByType<LayerSound>();
     for (LayerSound* layer : allSoundLayers)
     {
+        if (!layer->visible()) { continue; }
         layer->foreachKeyFrame([&allSoundClips](KeyFrame* key)
         {
             if (!key->fileName().isEmpty())
@@ -191,7 +199,7 @@ Status MovieExporter::assembleAudio(const Object* obj,
     QString filterComplex, amergeInput, panChannelLayout;
     QStringList args;
 
-    int wholeLen = qCeil((endFrame - startFrame) * 44100.0 / fps);
+    int wholeLen = qCeil(endFrame * 44100.0 / fps);
     for (auto clip : allSoundClips)
     {
         if (mCanceled)
@@ -215,11 +223,22 @@ Status MovieExporter::assembleAudio(const Object* obj,
     panChannelLayout.chop(1);
     // Output arguments
     // Mix audio
-    args << "-filter_complex" << QString("%1%2 amerge=inputs=%3, pan=mono|c0=%4 [out]")
-            .arg(filterComplex).arg(amergeInput).arg(clipCount).arg(panChannelLayout);
+    args << "-filter_complex";
+    if (clipCount == 1)
+    {
+        // If there is only one sound clip there is no need to use amerge
+        // Prior to ffmpeg 3.2, amerge does not support inputs=1
+        filterComplex.chop(1); // Remove final semicolon since there are no more filters added after
+        args << filterComplex << "-map" << amergeInput;
+    }
+    else {
+        args << QString("%1%2 amerge=inputs=%3, pan=mono|c0=%4 [out]")
+                .arg(filterComplex).arg(amergeInput).arg(clipCount).arg(panChannelLayout);
+        args << "-map" << "[out]";
+    }
     // Convert audio file: 44100Hz sampling rate, stereo, signed 16 bit little endian
     // Supported audio file types: wav, mp3, ogg... ( all file types supported by ffmpeg )
-    args << "-ar" << "44100" << "-acodec" << "pcm_s16le" << "-ac" << "2" << "-map" << "[out]" << "-y";
+    args << "-ar" << "44100" << "-acodec" << "pcm_s16le" << "-ac" << "2" << "-y";
     // Trim audio
     args << "-ss" << QString::number((startFrame - 1) / static_cast<double>(fps));
     args << "-to" << QString::number(endFrame / static_cast<double>(fps));
@@ -325,7 +344,7 @@ Status MovieExporter::generateMovie(
         args << "-plays" << (loop ? "0" : "1");
     }
 
-    if (strOutputFile.endsWith("mp4", Qt::CaseInsensitive))
+    if (strOutputFile.endsWith(".mp4", Qt::CaseInsensitive))
     {
         args << "-pix_fmt" << "yuv420p";
     }
@@ -340,7 +359,7 @@ Status MovieExporter::generateMovie(
 
     // Run FFmpeg command
 
-    STATUS_CHECK(executeFFMpegPipe(ffmpegPath, args, progress, [&](QProcess& ffmpeg, int framesProcessed)
+    Status status = executeFFMpegPipe(ffmpegPath, args, progress, [&](QProcess& ffmpeg, int framesProcessed)
     {
         if(framesProcessed < 0)
         {
@@ -365,10 +384,13 @@ Status MovieExporter::generateMovie(
             obj->paintImage(painter, currentFrame, false, true);
             painter.end();
 
-            // Should use sizeInBytes instead of byteCount to support large images,
-            // but this is only supported in QT 5.10+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+            int bytesWritten = ffmpeg.write(reinterpret_cast<const char*>(imageToExport.constBits()), imageToExport.sizeInBytes());
+            Q_ASSERT(bytesWritten == imageToExport.sizeInBytes());
+#else
             int bytesWritten = ffmpeg.write(reinterpret_cast<const char*>(imageToExport.constBits()), imageToExport.byteCount());
             Q_ASSERT(bytesWritten == imageToExport.byteCount());
+#endif
 
             currentFrame++;
             failCounter = 0;
@@ -376,7 +398,8 @@ Status MovieExporter::generateMovie(
         }
 
         return false;
-    }));
+    });
+    STATUS_CHECK(status);
 
     return Status::OK;
 }
@@ -456,7 +479,7 @@ Status MovieExporter::generateGif(
 
     // Run FFmpeg command
 
-    STATUS_CHECK(executeFFMpegPipe(ffmpegPath, args, progress, [&](QProcess& ffmpeg, int framesProcessed)
+    Status status = executeFFMpegPipe(ffmpegPath, args, progress, [&](QProcess& ffmpeg, int framesProcessed)
     {
         /* The GIF FFmpeg command requires the entires stream to be
          * written before FFmpeg can encode the GIF. This is because
@@ -482,13 +505,19 @@ Status MovieExporter::generateGif(
 
         obj->paintImage(painter, currentFrame, false, true);
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+        bytesWritten = ffmpeg.write(reinterpret_cast<const char*>(imageToExport.constBits()), imageToExport.sizeInBytes());
+        Q_ASSERT(bytesWritten == imageToExport.sizeInBytes());
+#else
         bytesWritten = ffmpeg.write(reinterpret_cast<const char*>(imageToExport.constBits()), imageToExport.byteCount());
         Q_ASSERT(bytesWritten == imageToExport.byteCount());
+#endif
 
         currentFrame++;
 
         return true;
-    }));
+    });
+    STATUS_CHECK(status);
 
     return Status::OK;
 }
@@ -529,7 +558,7 @@ Status MovieExporter::executeFFmpeg(const QString& cmd, const QStringList& args,
             if(!ffmpeg.waitForReadyRead()) break;
 
             QString output(ffmpeg.readAll());
-            QStringList sList = output.split(QRegExp("[\r\n]"), QString::SkipEmptyParts);
+            QStringList sList = output.split(QRegularExpression("[\r\n]"), SplitBehaviorFlags::SkipEmptyParts);
             for (const QString& s : sList)
             {
                 qDebug() << "[ffmpeg]" << s;
@@ -553,7 +582,7 @@ Status MovieExporter::executeFFmpeg(const QString& cmd, const QStringList& args,
         }
 
         QString output(ffmpeg.readAll());
-        QStringList sList = output.split(QRegExp("[\r\n]"), QString::SkipEmptyParts);
+        QStringList sList = output.split(QRegularExpression("[\r\n]"), SplitBehaviorFlags::SkipEmptyParts);
         for (const QString& s : sList)
         {
             qDebug() << "[ffmpeg]" << s;
@@ -563,8 +592,8 @@ Status MovieExporter::executeFFmpeg(const QString& cmd, const QStringList& args,
         if(ffmpeg.exitStatus() != QProcess::NormalExit || ffmpeg.exitCode() != 0)
         {
             status = Status::FAIL;
-            status.setTitle(QObject::tr("Something went wrong"));
-            status.setDescription(QObject::tr("Looks like our video backend did not exit normally. Your movie may not have exported correctly. Please try again and report this if it persists."));
+            status.setTitle(tr("Something went wrong"));
+            status.setDescription(tr("Looks like our video backend did not exit normally. Your movie may not have exported correctly. Please try again and report this if it persists."));
             dd << QString("Exit status: ").append(QProcess::NormalExit ? "NormalExit": "CrashExit")
                << QString("Exit code: %1").arg(ffmpeg.exitCode());
             status.setDetails(dd);
@@ -575,8 +604,8 @@ Status MovieExporter::executeFFmpeg(const QString& cmd, const QStringList& args,
     {
         qDebug() << "ERROR: Could not execute FFmpeg.";
         status = Status::FAIL;
-        status.setTitle(QObject::tr("Something went wrong"));
-        status.setDescription(QObject::tr("Couldn't start the video backend, please try again."));
+        status.setTitle(tr("Something went wrong"));
+        status.setDescription(tr("Couldn't start the video backend, please try again."));
         status.setDetails(dd);
     }
     return status;
@@ -659,7 +688,7 @@ Status MovieExporter::executeFFMpegPipe(const QString& cmd, const QStringList& a
             if(ffmpeg.waitForReadyRead(10))
             {
                 QString output(ffmpeg.readAll());
-                QStringList sList = output.split(QRegExp("[\r\n]"), QString::SkipEmptyParts);
+                QStringList sList = output.split(QRegularExpression("[\r\n]"), SplitBehaviorFlags::SkipEmptyParts);
                 for (const QString& s : sList)
                 {
                     qDebug() << "[ffmpeg]" << s;
@@ -690,7 +719,7 @@ Status MovieExporter::executeFFMpegPipe(const QString& cmd, const QStringList& a
         }
 
         QString output(ffmpeg.readAll());
-        QStringList sList = output.split(QRegExp("[\r\n]"), QString::SkipEmptyParts);
+        QStringList sList = output.split(QRegularExpression("[\r\n]"), SplitBehaviorFlags::SkipEmptyParts);
         for (const QString& s : sList)
         {
             qDebug() << "[ffmpeg]" << s;
@@ -700,8 +729,8 @@ Status MovieExporter::executeFFMpegPipe(const QString& cmd, const QStringList& a
         if(ffmpeg.exitStatus() != QProcess::NormalExit  || ffmpeg.exitCode() != 0)
         {
             status = Status::FAIL;
-            status.setTitle(QObject::tr("Something went wrong"));
-            status.setDescription(QObject::tr("Looks like our video backend did not exit normally. Your movie may not have exported correctly. Please try again and report this if it persists."));
+            status.setTitle(tr("Something went wrong"));
+            status.setDescription(tr("Looks like our video backend did not exit normally. Your movie may not have exported correctly. Please try again and report this if it persists."));
             dd << QString("Exit status: ").append(QProcess::NormalExit ? "NormalExit": "CrashExit")
                << QString("Exit code: %1").arg(ffmpeg.exitCode());
             status.setDetails(dd);
@@ -712,8 +741,8 @@ Status MovieExporter::executeFFMpegPipe(const QString& cmd, const QStringList& a
     {
         qDebug() << "ERROR: Could not execute FFmpeg.";
         status = Status::FAIL;
-        status.setTitle(QObject::tr("Something went wrong"));
-        status.setDescription(QObject::tr("Couldn't start the video backend, please try again."));
+        status.setTitle(tr("Something went wrong"));
+        status.setDescription(tr("Couldn't start the video backend, please try again."));
         status.setDetails(dd);
     }
 
