@@ -29,6 +29,7 @@ KeyFrame::KeyFrame(const KeyFrame& k2)
     mIsModified = k2.mIsModified;
     mIsSelected = k2.mIsSelected;
     mAttachedFileName = k2.mAttachedFileName;
+    // intentionally not copying event listeners
 }
 
 KeyFrame::~KeyFrame()
@@ -39,6 +40,22 @@ KeyFrame::~KeyFrame()
     }
 }
 
+KeyFrame& KeyFrame::operator=(const KeyFrame& k2)
+{
+	if (this == &k2)
+	{
+		return *this; // a self-assignment
+	}
+
+	mFrame = k2.mFrame;
+	mLength = k2.mLength;
+	mIsModified = k2.mIsModified;
+	mIsSelected = k2.mIsSelected;
+	mAttachedFileName = k2.mAttachedFileName;
+    // intentionally not copying event listeners
+    return *this;
+}
+
 void KeyFrame::addEventListener(KeyFrameEventListener* listener)
 {
     auto it = std::find(mEventListeners.begin(), mEventListeners.end(), listener);
