@@ -23,23 +23,15 @@ GNU General Public License for more details.
 #include <QString>
 #include <QDomElement>
 #include "pencilerror.h"
-#include "pencildef.h"
-
-class QMouseEvent;
-class QPainter;
 
 class KeyFrame;
-class Object;
-class TimeLineCells;
 class Status;
 
-#define ProgressCallback std::function<void()>
+typedef std::function<void()> ProgressCallback;
 
-
-class Layer : public QObject
+class Layer
 {
-    Q_OBJECT
-
+    Q_DECLARE_TR_FUNCTIONS(Layer)
 public:
     enum LAYER_TYPE
     {
@@ -51,15 +43,13 @@ public:
         CAMERA = 5,
     };
 
-    explicit Layer(Object*, LAYER_TYPE);
-    ~Layer() override;
+    explicit Layer(int id, LAYER_TYPE eType);
+    virtual ~Layer();
 
     int id() const { return mId; }
+    void setId(int layerId) { mId = layerId; }
     LAYER_TYPE type() const { return meType; }
 
-    Object* object() const { return mObject; }
-    void setObject(Object* obj);
-
     void setName(QString name) { mName = name; }
     QString name() const { return mName; }
 
@@ -68,6 +58,12 @@ public:
     bool visible() const { return mVisible; }
     void setVisible(bool b) { mVisible = b; }
 
+    /** Get selected keyframe positions sorted by position */
+    QList<int> selectedKeyFramesPositions() const { return mSelectedFrames_byPosition; }
+
+    /** Get selected keyframe positions based on the order they were selected */
+    QList<int> selectedKeyFramesByLast() const { return mSelectedFrames_byLast; }
+
     virtual Status saveKeyFrameFile(KeyFrame*, QString dataPath) = 0;
     virtual void loadDomElement(const QDomElement& element, QString dataDirPath, ProgressCallback progressForward) = 0;
     virtual QDomElement createDomElement(QDomDocument& doc) const = 0;
@@ -85,22 +81,39 @@ public:
     int  getNextFrameNumber(int position, bool isAbsolute) const;
 
     int keyFrameCount() const { return static_cast<int>(mKeyFrames.size()); }
+    int selectedKeyFrameCount() const { return mSelectedFrames_byPosition.count(); }
+    bool hasAnySelectedFrames() const { return !mSelectedFrames_byLast.empty() && !mSelectedFrames_byPosition.empty(); }
 
+    /** Will insert an empty frame (exposure) after the given position
+        @param position The frame to add exposure to
+    */
+    bool insertExposureAt(int position);
+
+    /**
+     * Creates a new keyframe at the given position, unless one already exists.
+     * @param position The position of the new keyframe
+     * @return false if a keyframe already exists at the position, true if the new keyframe was successfully added
+     */
     bool addNewKeyFrameAt(int position);
-    bool addKeyFrame(int position, KeyFrame*);
-    bool removeKeyFrame(int position);
+    void addOrReplaceKeyFrame(int position, KeyFrame* pKeyFrame);
+    /**
+     * Adds a keyframe at the given position, unless one already exists.
+     * @param position The new position of the keyframe
+     * @param pKeyFrame The keyframe to add. Its previous position will be overwritten
+     * @return false if a keyframe already exists at the position, true if the keyframe was successfully added
+     */
+    virtual bool addKeyFrame(int position, KeyFrame* pKeyFrame);
+    virtual bool removeKeyFrame(int position);
     bool swapKeyFrames(int position1, int position2);
     bool moveKeyFrame(int position, int offset);
-    bool loadKey(KeyFrame*);
     KeyFrame* getKeyFrameAt(int position) const;
     KeyFrame* getLastKeyFrameAtPosition(int position) const;
     bool keyExistsWhichCovers(int frameNumber);
     KeyFrame *getKeyFrameWhichCovers(int frameNumber) const;
-    bool getVisibility() const { return mVisible; }
 
     void foreachKeyFrame(std::function<void(KeyFrame*)>) const;
 
-    void setModified(int position, bool isModified);
+    void setModified(int position, bool isModified) const;
 
     // Handle selection
     bool isFrameSelected(int position) const;
@@ -108,9 +121,39 @@ public:
     void toggleFrameSelected(int position, bool allowMultiple = false);
     void extendSelectionTo(int position);
     void selectAllFramesAfter(int position);
+
+    /** Make a selection from specified position until a blank spot appears
+     *  The search is only looking forward, e.g.
+     *  @code
+     *  |123| 4 5
+     *   ^
+     *   pos/search from
+     *  @endcode
+     *  @param position the current position
+     */
+    bool newSelectionOfConnectedFrames(int position);
+
+    /** Add or subtract exposure from selected frames
+     * @param offset Any value above 0 for adding exposure and any value below 0 to subtract exposure
+     */
+    void setExposureForSelectedFrames(int offset);
+
+    /** Reverse order of selected frames
+     * @return true if all frames were successfully reversed, otherwise will return false.
+     */
+    bool reverseOrderOfSelection();
+
     void deselectAll();
 
     bool moveSelectedFrames(int offset);
+    QList<int> getSelectedFramesByPos() const { return mSelectedFrames_byPosition; }
+
+    /** Predetermines whether the frames can be moved to a new position depending on the offset
+     *
+     * @param offset Should be start press position - current position
+     * @return true if selected frames can be moved otherwise false
+     */
+    bool canMoveSelectedFramesToOffset(int offset) const;
 
     Status save(const QString& sDataFolder, QStringList& attachedFiles, ProgressCallback progressStep);
     virtual Status presave(const QString& sDataFolder) { Q_UNUSED(sDataFolder); return Status::SAFE; }
@@ -127,14 +170,14 @@ public:
     /** Clear the list of dirty keyframes */
     void clearDirtyFrames() { mDirtyFrames.clear(); }
 
-
 protected:
-    void setId(int LayerId) { mId = LayerId; }
-    virtual KeyFrame* createKeyFrame(int position, Object*) = 0;
+    virtual KeyFrame* createKeyFrame(int position) = 0;
+    bool loadKey(KeyFrame*);
 
 private:
+    void removeFromSelectionList(int position);
+
     LAYER_TYPE meType = UNDEFINED;
-    Object*    mObject = nullptr;
     int        mId = 0;
     bool       mVisible = true;
     QString    mName;
