@@ -21,11 +21,9 @@ GNU General Public License for more details.
 #include <QFile>
 #include "keyframe.h"
 #include "bitmapimage.h"
+#include "util/util.h"
 
-
-
-
-LayerBitmap::LayerBitmap(Object* object) : Layer(object, Layer::BITMAP)
+LayerBitmap::LayerBitmap(int id) : Layer(id, Layer::BITMAP)
 {
     setName(tr("Bitmap Layer"));
 }
@@ -46,11 +44,26 @@ BitmapImage* LayerBitmap::getLastBitmapImageAtFrame(int frameNumber, int increme
     return static_cast<BitmapImage*>(getLastKeyFrameAtPosition(frameNumber + increment));
 }
 
-void LayerBitmap::loadImageAtFrame(QString path, QPoint topLeft, int frameNumber)
+void LayerBitmap::repositionFrame(QPoint point, int frame)
+{
+    BitmapImage* image = getBitmapImageAtFrame(frame);
+    Q_ASSERT(image);
+    image->moveTopLeft(point);
+}
+
+QRect LayerBitmap::getFrameBounds(int frame)
+{
+    BitmapImage* image = getBitmapImageAtFrame(frame);
+    Q_ASSERT(image);
+    return image->bounds();
+}
+
+void LayerBitmap::loadImageAtFrame(QString path, QPoint topLeft, int frameNumber, qreal opacity)
 {
     BitmapImage* pKeyFrame = new BitmapImage(topLeft, path);
     pKeyFrame->enableAutoCrop(true);
     pKeyFrame->setPos(frameNumber);
+    pKeyFrame->setOpacity(opacity);
     loadKey(pKeyFrame);
 }
 
@@ -86,7 +99,7 @@ Status LayerBitmap::saveKeyFrameFile(KeyFrame* keyframe, QString path)
     return Status::OK;
 }
 
-KeyFrame* LayerBitmap::createKeyFrame(int position, Object*)
+KeyFrame* LayerBitmap::createKeyFrame(int position)
 {
     BitmapImage* b = new BitmapImage;
     b->setPos(position);
@@ -172,6 +185,7 @@ QDomElement LayerBitmap::createDomElement(QDomDocument& doc) const
         imageTag.setAttribute("src", fileName(pKeyFrame));
         imageTag.setAttribute("topLeftX", pImg->topLeft().x());
         imageTag.setAttribute("topLeftY", pImg->topLeft().y());
+        imageTag.setAttribute("opacity", pImg->getOpacity());
         layerElem.appendChild(imageTag);
 
         Q_ASSERT(QFileInfo(pKeyFrame->fileName()).fileName() == fileName(pKeyFrame));
@@ -188,20 +202,19 @@ void LayerBitmap::loadDomElement(const QDomElement& element, QString dataDirPath
     while (!imageTag.isNull())
     {
         QDomElement imageElement = imageTag.toElement();
-        if (!imageElement.isNull())
+        if (!imageElement.isNull() && imageElement.tagName() == "image")
         {
-            if (imageElement.tagName() == "image")
+            QString path = validateDataPath(imageElement.attribute("src"), dataDirPath);
+            if (!path.isEmpty())
             {
-                QString path = dataDirPath + "/" + imageElement.attribute("src"); // the file is supposed to be in the data directory
-                QFileInfo fi(path);
-                if (!fi.exists()) path = imageElement.attribute("src");
                 int position = imageElement.attribute("frame").toInt();
                 int x = imageElement.attribute("topLeftX").toInt();
                 int y = imageElement.attribute("topLeftY").toInt();
-                loadImageAtFrame(path, QPoint(x, y), position);
-
-                progressStep();
+                qreal opacity = imageElement.attribute("opacity", "1.0").toDouble();
+                loadImageAtFrame(path, QPoint(x, y), position, opacity);
             }
+
+            progressStep();
         }
         imageTag = imageTag.nextSibling();
     }
