@@ -20,9 +20,9 @@ GNU General Public License for more details.
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
+#include "util/util.h"
 
-
-LayerVector::LayerVector(Object* object) : Layer(object, Layer::VECTOR)
+LayerVector::LayerVector(int id) : Layer(id, Layer::VECTOR)
 {
     setName(tr("Vector Layer"));
 }
@@ -70,7 +70,6 @@ void LayerVector::loadImageAtFrame(QString path, int frameNumber)
     }
     VectorImage* vecImg = new VectorImage;
     vecImg->setPos(frameNumber);
-    vecImg->setObject(object());
     vecImg->read(path);
     addKeyFrame(frameNumber, vecImg);
 }
@@ -106,11 +105,10 @@ Status LayerVector::saveKeyFrameFile(KeyFrame* keyFrame, QString path)
     return Status::OK;
 }
 
-KeyFrame* LayerVector::createKeyFrame(int position, Object* obj)
+KeyFrame* LayerVector::createKeyFrame(int position)
 {
     VectorImage* v = new VectorImage;
     v->setPos(position);
-    v->setObject(obj);
     return v;
 }
 
@@ -139,6 +137,8 @@ QDomElement LayerVector::createDomElement(QDomDocument& doc) const
         QDomElement imageTag = doc.createElement("image");
         imageTag.setAttribute("frame", keyframe->pos());
         imageTag.setAttribute("src", fileName(keyframe));
+        VectorImage* image = getVectorImageAtFrame(keyframe->pos());
+        imageTag.setAttribute("opacity", image->getOpacity());
         layerElem.appendChild(imageTag);
 
         Q_ASSERT(QFileInfo(keyframe->fileName()).fileName() == fileName(keyframe));
@@ -155,26 +155,31 @@ void LayerVector::loadDomElement(const QDomElement& element, QString dataDirPath
     while (!imageTag.isNull())
     {
         QDomElement imageElement = imageTag.toElement();
-        if (!imageElement.isNull())
+        if (!imageElement.isNull() && imageElement.tagName() == "image")
         {
-            if (imageElement.tagName() == "image")
+            int position;
+            QString rawPath = imageElement.attribute("src");
+            if (!rawPath.isNull())
             {
-                if (!imageElement.attribute("src").isNull())
+                QString path = validateDataPath(rawPath, dataDirPath);
+                if (!path.isEmpty())
                 {
-                    QString path = dataDirPath + "/" + imageElement.attribute("src"); // the file is supposed to be in the data directory
-                    QFileInfo fi(path);
-                    if (!fi.exists()) path = imageElement.attribute("src");
-                    int position = imageElement.attribute("frame").toInt();
+                    position = imageElement.attribute("frame").toInt();
                     loadImageAtFrame(path, position);
+                    getVectorImageAtFrame(position)->setOpacity(imageElement.attribute("opacity", "1.0").toDouble());
                 }
-                else
-                {
-                    int frame = imageElement.attribute("frame").toInt();
-                    addNewKeyFrameAt(frame);
-                    getVectorImageAtFrame(frame)->loadDomElement(imageElement);
-                }
-                progressStep();
             }
+            else
+            {
+                position = imageElement.attribute("frame").toInt();
+                addNewKeyFrameAt(position);
+                getVectorImageAtFrame(position)->loadDomElement(imageElement);
+                getVectorImageAtFrame(position)->setOpacity(imageElement.attribute("opacity", "1.0").toDouble());
+            }
+
+
+
+            progressStep();
         }
         imageTag = imageTag.nextSibling();
     }
