@@ -48,8 +48,10 @@ SoundClip& SoundClip::operator=(const SoundClip& a)
     return *this;
 }
 
-SoundClip* SoundClip::clone()
+SoundClip* SoundClip::clone() const
 {
+    // Question: need to copy the file?
+    // The audio files are not allowed to be edited in Pencil2D, it should be file for now.
     return new SoundClip(*this);
 }
 
@@ -70,7 +72,7 @@ bool SoundClip::isValid() const
         return false;
     }
 
-    if (mPlayer == nullptr)
+    if (!mPlayer)
     {
         return false;
     }
