@@ -18,16 +18,15 @@ GNU General Public License for more details.
 
 #include <QPixmap>
 #include <QPainter>
-#include <QtMath>
 #include <QSettings>
 #include "pointerevent.h"
 
 #include "layer.h"
 #include "layervector.h"
 #include "layerbitmap.h"
+#include "layercamera.h"
 #include "layermanager.h"
 #include "colormanager.h"
-#include "strokemanager.h"
 #include "viewmanager.h"
 #include "vectorimage.h"
 #include "editor.h"
@@ -47,6 +46,7 @@ void BucketTool::loadSettings()
 {
     mPropertyEnabled[TOLERANCE] = true;
     mPropertyEnabled[WIDTH] = true;
+    mPropertyEnabled[FILL_MODE] = true;
 
     QSettings settings(PENCIL2D, PENCIL2D);
 
@@ -54,31 +54,49 @@ void BucketTool::loadSettings()
     properties.feather = 10;
     properties.stabilizerLevel = StabilizationLevel::NONE;
     properties.useAA = DISABLED;
-    properties.tolerance = settings.value("tolerance", 32.0).toDouble();
+    properties.tolerance = settings.value(SETTING_BUCKET_TOLERANCE, 32.0).toDouble();
+    properties.toleranceEnabled = settings.value(SETTING_BUCKET_TOLERANCE_ON, false).toBool();
+
+    properties.bucketFillExpand = settings.value(SETTING_BUCKET_FILL_EXPAND, 2.0).toInt();
+    properties.bucketFillExpandEnabled = settings.value(SETTING_BUCKET_FILL_EXPAND_ON, true).toBool();
+    properties.bucketFillReferenceMode = settings.value(SETTING_BUCKET_FILL_REFERENCE_MODE, 0).toInt();
+    properties.fillMode = settings.value(SETTING_FILL_MODE, 0).toInt();
 }
 
 void BucketTool::resetToDefault()
 {
     setWidth(4.0);
     setTolerance(32.0);
+    setFillMode(0);
+    setFillExpand(2);
+    setFillExpandEnabled(true);
+    setToleranceEnabled(false);
+    setFillReferenceMode(0);
 }
 
 QCursor BucketTool::cursor()
 {
     if (mEditor->preference()->isOn(SETTING::TOOL_CURSOR))
     {
-        QPixmap pixmap(":icons/bucketTool.png");
-        QPainter painter(&pixmap);
-        painter.end();
-
-        return QCursor(pixmap, 4, 20);
+        return QCursor(QPixmap(":icons/general/cursor-bucket.svg"), -1, 17);
     }
     else
     {
-        return QCursor(QPixmap(":icons/cross.png"), 10, 10);
+        return QCursor(QPixmap(":icons/general/cross.png"), 10, 10);
     }
 }
 
+void BucketTool::setTolerance(const int tolerance)
+{
+    // Set current property
+    properties.tolerance = tolerance;
+
+    // Update settings
+    QSettings settings(PENCIL2D, PENCIL2D);
+    settings.setValue(SETTING_BUCKET_TOLERANCE, tolerance);
+    settings.sync();
+}
+
 /**
  * @brief BrushTool::setWidth
  * @param width
@@ -95,24 +113,83 @@ void BucketTool::setWidth(const qreal width)
     settings.sync();
 }
 
-void BucketTool::setTolerance(const int tolerance)
+void BucketTool::setFillMode(int mode)
 {
     // Set current property
-    properties.tolerance = tolerance;
+    properties.fillMode = mode;
 
     // Update settings
     QSettings settings(PENCIL2D, PENCIL2D);
-    settings.setValue("tolerance", tolerance);
+    settings.setValue(SETTING_FILL_MODE, mode);
+    settings.sync();
+}
+
+void BucketTool::setToleranceEnabled(const bool enabled)
+{
+    properties.toleranceEnabled = enabled;
+
+    // Update settings
+    QSettings settings(PENCIL2D, PENCIL2D);
+    settings.setValue(SETTING_BUCKET_TOLERANCE_ON, enabled);
+    settings.sync();
+}
+
+void BucketTool::setFillExpandEnabled(const bool enabled)
+{
+    properties.bucketFillExpandEnabled = enabled;
+
+    // Update settings
+    QSettings settings(PENCIL2D, PENCIL2D);
+    settings.setValue(SETTING_BUCKET_FILL_EXPAND_ON, enabled);
+    settings.sync();
+}
+
+void BucketTool::setFillExpand(const int fillExpandValue)
+{
+    properties.bucketFillExpand = fillExpandValue;
+
+    // Update settings
+    QSettings settings(PENCIL2D, PENCIL2D);
+    settings.setValue(SETTING_BUCKET_FILL_EXPAND, fillExpandValue);
+    settings.sync();
+}
+
+void BucketTool::setFillReferenceMode(int referenceMode)
+{
+    properties.bucketFillReferenceMode = referenceMode;
+
+    // Update settings
+    QSettings settings(PENCIL2D, PENCIL2D);
+    settings.setValue(SETTING_BUCKET_FILL_REFERENCE_MODE, referenceMode);
     settings.sync();
 }
 
 void BucketTool::pointerPressEvent(PointerEvent* event)
 {
+    mInterpolator.pointerPressEvent(event);
     startStroke(event->inputType());
+
+    Layer* targetLayer = mEditor->layers()->currentLayer();
+
+    if (targetLayer->type() != Layer::BITMAP) { return; }
+
+    LayerCamera* layerCam = mEditor->layers()->getCameraLayerBelow(mEditor->currentLayerIndex());
+
+    mBitmapBucket = BitmapBucket(mEditor,
+                                 mEditor->color()->frontColor(),
+                                 layerCam ? layerCam->getViewAtFrame(mEditor->currentFrame()).inverted().mapRect(layerCam->getViewRect()) : QRect(),
+                                 getCurrentPoint(),
+                                 properties);
+
+    // Because we can change layer to on the fly, but we do not act reactively
+    // on it, it's necessary to invalidate layer cache on press event.
+    // Otherwise, the cache will be drawn until a move event has been initiated.
+    mScribbleArea->invalidatePainterCaches();
 }
 
 void BucketTool::pointerMoveEvent(PointerEvent* event)
 {
+    mInterpolator.pointerMoveEvent(event);
     if (event->buttons() & Qt::LeftButton && event->inputType() == mCurrentInputType)
     {
         Layer* layer = mEditor->layers()->currentLayer();
@@ -120,11 +197,17 @@ void BucketTool::pointerMoveEvent(PointerEvent* event)
         {
             drawStroke();
         }
+        else if (layer->type() == Layer::BITMAP)
+        {
+            paintBitmap();
+            mFilledOnMove = true;
+        }
     }
 }
 
 void BucketTool::pointerReleaseEvent(PointerEvent* event)
 {
+    mInterpolator.pointerReleaseEvent(event);
     if (event->inputType() != mCurrentInputType) return;
 
     Layer* layer = editor()->layers()->currentLayer();
@@ -132,55 +215,39 @@ void BucketTool::pointerReleaseEvent(PointerEvent* event)
 
     if (event->button() == Qt::LeftButton)
     {
-        mEditor->backup(typeName());
-
-        switch (layer->type())
+        // Backup of bitmap image is more complicated now and has therefore been moved to bitmap code
+        if (layer->type() == Layer::VECTOR) {
+            mEditor->backup(typeName());
+            paintVector(layer);
+        }
+        else if (layer->type() == Layer::BITMAP && !mFilledOnMove)
         {
-        case Layer::BITMAP: paintBitmap(layer); break;
-        case Layer::VECTOR: paintVector(layer); break;
-        default:
-            break;
+            paintBitmap();
         }
     }
+    mFilledOnMove = false;
+
     endStroke();
 }
 
-bool BucketTool::startAdjusting(Qt::KeyboardModifiers modifiers, qreal argStep)
+void BucketTool::paintBitmap()
 {
-    mQuickSizingProperties.clear();
-    if (mEditor->layers()->currentLayer()->type() == Layer::VECTOR)
+    mBitmapBucket.paint(getCurrentPoint(), [this](BucketState progress, int layerIndex, int frameIndex)
     {
-        mQuickSizingProperties.insert(Qt::ShiftModifier, WIDTH);
-    }
-    else
-    {
-        mQuickSizingProperties.insert(Qt::ControlModifier, TOLERANCE);
-    }
-    return BaseTool::startAdjusting(modifiers, argStep);
-}
-
-void BucketTool::paintBitmap(Layer* layer)
-{
-    Layer* targetLayer = layer; // by default
-    int layerNumber = editor()->layers()->currentLayerIndex(); // by default
-
-    BitmapImage* targetImage = static_cast<LayerBitmap*>(targetLayer)->getLastBitmapImageAtFrame(editor()->currentFrame(), 0);
-    if (targetImage == nullptr) { return; } // Can happen if the first frame is deleted while drawing
-
-    QPoint point = QPoint(qFloor(getLastPoint().x()), qFloor(getLastPoint().y()));
-    QRect cameraRect = mScribbleArea->getCameraRect().toRect();
-    BitmapImage::floodFill(targetImage,
-                           cameraRect,
-                           point,
-                           qPremultiply(mEditor->color()->frontColor().rgba()),
-                           properties.tolerance);
-
-    mScribbleArea->setModified(layerNumber, mEditor->currentFrame());
+        if (progress == BucketState::WillFillTarget)
+        {
+            mEditor->backup(layerIndex, frameIndex, typeName());
+        }
+        else if (progress == BucketState::DidFillTarget)
+        {
+            mEditor->setModified(layerIndex, frameIndex);
+        }
+    });
 }
 
 void BucketTool::paintVector(Layer* layer)
 {
-    mScribbleArea->clearBitmapBuffer();
+    mScribbleArea->clearDrawingBuffer();
 
     VectorImage* vectorImage = static_cast<LayerVector*>(layer)->getLastVectorImageAtFrame(mEditor->currentFrame(), 0);
     if (vectorImage == nullptr) { return; } // Can happen if the first frame is deleted while drawing
@@ -196,7 +263,7 @@ void BucketTool::paintVector(Layer* layer)
 
     applyChanges();
 
-    mScribbleArea->setModified(mEditor->layers()->currentLayerIndex(), mEditor->currentFrame());
+    mEditor->setModified(mEditor->layers()->currentLayerIndex(), mEditor->currentFrame());
 }
 
 void BucketTool::applyChanges()
@@ -208,22 +275,19 @@ void BucketTool::drawStroke()
 {
     StrokeTool::drawStroke();
 
-    if (properties.stabilizerLevel != strokeManager()->getStabilizerLevel())
+    if (properties.stabilizerLevel != mInterpolator.getStabilizerLevel())
     {
-        strokeManager()->setStabilizerLevel(properties.stabilizerLevel);
+        mInterpolator.setStabilizerLevel(properties.stabilizerLevel);
     }
 
-    QList<QPointF> p = strokeManager()->interpolateStroke();
+    QList<QPointF> p = mInterpolator.interpolateStroke();
 
     Layer* layer = mEditor->layers()->currentLayer();
 
     if (layer->type() == Layer::VECTOR)
     {
         mCurrentWidth = 30;
-        int rad = qRound((mCurrentWidth / 2 + 2) * mEditor->view()->scaling());
-
         QColor pathColor = qPremultiply(mEditor->color()->frontColor().rgba());
-        //pathColor.setAlpha(255);
 
         QPen pen(pathColor,
                  mCurrentWidth * mEditor->view()->scaling(),
@@ -236,7 +300,6 @@ void BucketTool::drawStroke()
             QPainterPath path(p[0]);
             path.cubicTo(p[1], p[2], p[3]);
             mScribbleArea->drawPath(path, pen, Qt::NoBrush, QPainter::CompositionMode_Source);
-            mScribbleArea->refreshVector(path.boundingRect().toRect(), rad);
         }
     }
 }
