@@ -19,6 +19,7 @@ GNU General Public License for more details.
 #define HANDTOOL_H
 
 #include "basetool.h"
+#include "preferencemanager.h"
 
 
 class HandTool : public BaseTool
@@ -35,14 +36,14 @@ public:
     void pointerMoveEvent(PointerEvent *) override;
     void pointerDoubleClickEvent(PointerEvent *) override;
 
-    virtual bool isActive() override { return false; }
-
 private:
-    void transformView(Qt::KeyboardModifiers keyMod, Qt::MouseButtons buttons);
+    void updateSettings(const SETTING setting);
+    void transformView(Qt::KeyboardModifiers keyMod, const QPointF& pos, Qt::MouseButtons buttons);
 
     QPointF mLastPixel;
     QPointF mStartPoint;
     bool mIsHeld = false;
+    int mDeltaFactor = 1;
 };
 
 #endif
