@@ -21,7 +21,6 @@ GNU General Public License for more details.
 #include "editor.h"
 #include "scribblearea.h"
 
-#include "strokemanager.h"
 #include "layermanager.h"
 #include "colormanager.h"
 #include "viewmanager.h"
@@ -31,7 +30,7 @@ GNU General Public License for more details.
 #include "vectorimage.h"
 
 
-PolylineTool::PolylineTool(QObject* parent) : BaseTool(parent)
+PolylineTool::PolylineTool(QObject* parent) : StrokeTool(parent)
 {
 }
 
@@ -42,6 +41,8 @@ ToolType PolylineTool::type()
 
 void PolylineTool::loadSettings()
 {
+    StrokeTool::loadSettings();
+
     mPropertyEnabled[WIDTH] = true;
     mPropertyEnabled[BEZIER] = true;
     mPropertyEnabled[ANTI_ALIASING] = true;
@@ -55,6 +56,8 @@ void PolylineTool::loadSettings()
     properties.preserveAlpha = OFF;
     properties.useAA = settings.value("brushAA").toBool();
     properties.stabilizerLevel = -1;
+
+    mQuickSizingProperties.insert(Qt::ShiftModifier, WIDTH);
 }
 
 void PolylineTool::resetToDefault()
@@ -91,23 +94,47 @@ void PolylineTool::setAA(const int AA)
     settings.sync();
 }
 
-bool PolylineTool::isActive()
+bool PolylineTool::leavingThisTool()
+{
+    StrokeTool::leavingThisTool();
+    if (mPoints.size() > 0)
+    {
+        cancelPolyline();
+    }
+    return true;
+}
+
+bool PolylineTool::isActive() const
 {
     return !mPoints.isEmpty();
 }
 
 QCursor PolylineTool::cursor()
 {
-    return QCursor(QPixmap(":icons/cross.png"), 10, 10);
+    return QCursor(QPixmap(":icons/general/cross.png"), 10, 10);
 }
 
 void PolylineTool::clearToolData()
 {
+    if (mPoints.empty()) {
+        return;
+    }
+
     mPoints.clear();
+    emit isActiveChanged(POLYLINE, false);
+
+    // Clear the in-progress polyline from the bitmap buffer.
+    mScribbleArea->clearDrawingBuffer();
+    mScribbleArea->updateFrame();
 }
 
 void PolylineTool::pointerPressEvent(PointerEvent* event)
 {
+    mInterpolator.pointerPressEvent(event);
+    if (handleQuickSizing(event)) {
+        return;
+    }
+
     Layer* layer = mEditor->layers()->currentLayer();
 
     if (event->button() == Qt::LeftButton)
@@ -127,31 +154,48 @@ void PolylineTool::pointerPressEvent(PointerEvent* event)
                 }
             }
             mPoints << getCurrentPoint();
+            emit isActiveChanged(POLYLINE, true);
         }
     }
+
+    StrokeTool::pointerPressEvent(event);
 }
 
-void PolylineTool::pointerMoveEvent(PointerEvent*)
+void PolylineTool::pointerMoveEvent(PointerEvent* event)
 {
+    mInterpolator.pointerMoveEvent(event);
+    if (handleQuickSizing(event)) {
+        return;
+    }
+
     Layer* layer = mEditor->layers()->currentLayer();
     if (layer->type() == Layer::BITMAP || layer->type() == Layer::VECTOR)
     {
         drawPolyline(mPoints, getCurrentPoint());
     }
+
+    StrokeTool::pointerMoveEvent(event);
 }
 
-void PolylineTool::pointerReleaseEvent(PointerEvent *)
-{}
-
-void PolylineTool::pointerDoubleClickEvent(PointerEvent*)
+void PolylineTool::pointerReleaseEvent(PointerEvent* event)
 {
+    mInterpolator.pointerReleaseEvent(event);
+    if (handleQuickSizing(event)) {
+        return;
+    }
+
+    StrokeTool::pointerReleaseEvent(event);
+}
+
+void PolylineTool::pointerDoubleClickEvent(PointerEvent* event)
+{
+    mInterpolator.pointerPressEvent(event);
     // include the current point before ending the line.
     mPoints << getCurrentPoint();
 
     mEditor->backup(typeName());
 
     endPolyline(mPoints);
-    clearToolData();
 }
 
 
@@ -163,7 +207,6 @@ bool PolylineTool::keyPressEvent(QKeyEvent* event)
         if (mPoints.size() > 0)
         {
             endPolyline(mPoints);
-            clearToolData();
             return true;
         }
         break;
@@ -172,16 +215,15 @@ bool PolylineTool::keyPressEvent(QKeyEvent* event)
         if (mPoints.size() > 0)
         {
             cancelPolyline();
-            clearToolData();
             return true;
         }
         break;
 
     default:
-        return false;
+        break;
     }
 
-    return false;
+    return BaseTool::keyPressEvent(event);
 }
 
 void PolylineTool::drawPolyline(QList<QPointF> points, QPointF endPoint)
@@ -212,7 +254,6 @@ void PolylineTool::drawPolyline(QList<QPointF> points, QPointF endPoint)
         {
             if (mEditor->layers()->currentLayer()->type() == Layer::VECTOR)
             {
-                tempPath = mEditor->view()->mapCanvasToScreen(tempPath);
                 if (mScribbleArea->makeInvisible() == true)
                 {
                     pen.setWidth(0);
@@ -220,7 +261,7 @@ void PolylineTool::drawPolyline(QList<QPointF> points, QPointF endPoint)
                 }
                 else
                 {
-                    pen.setWidth(properties.width * mEditor->view()->scaling());
+                    pen.setWidth(properties.width);
                 }
             }
         }
@@ -232,15 +273,12 @@ void PolylineTool::drawPolyline(QList<QPointF> points, QPointF endPoint)
 
 void PolylineTool::cancelPolyline()
 {
-    // Clear the in-progress polyline from the bitmap buffer.
-    mScribbleArea->clearBitmapBuffer();
-    mScribbleArea->updateCurrentFrame();
+    clearToolData();
 }
 
 void PolylineTool::endPolyline(QList<QPointF> points)
 {
     Layer* layer = mEditor->layers()->currentLayer();
-    mScribbleArea->clearBitmapBuffer();
 
     if (layer->type() == Layer::VECTOR)
     {
@@ -264,11 +302,9 @@ void PolylineTool::endPolyline(QList<QPointF> points)
     if (layer->type() == Layer::BITMAP)
     {
         drawPolyline(points, points.last());
-        BitmapImage *bitmapImage = static_cast<LayerBitmap*>(layer)->getLastBitmapImageAtFrame(mEditor->currentFrame(), 0);
-        if (bitmapImage == nullptr) { return; } // Can happen if the first frame is deleted while drawing
-        bitmapImage->paste(mScribbleArea->mBufferImg);
     }
+    mScribbleArea->endStroke();
+    mEditor->setModified(mEditor->layers()->currentLayerIndex(), mEditor->currentFrame());
 
-    mScribbleArea->clearBitmapBuffer();
-    mScribbleArea->setModified(mEditor->layers()->currentLayerIndex(), mEditor->currentFrame());
+    clearToolData();
 }
