@@ -20,9 +20,9 @@ GNU General Public License for more details.
 
 #include <QPointF>
 
-#include "basetool.h"
+#include "stroketool.h"
 
-class PolylineTool : public BaseTool
+class PolylineTool : public StrokeTool
 {
     Q_OBJECT
 public:
@@ -45,7 +45,9 @@ public:
     void setFeather(const qreal feather) override;
     void setAA(const int AA) override;
 
-    virtual bool isActive() override;
+    bool leavingThisTool() override;
+
+    bool isActive() const override;
 
 private:
     QList<QPointF> mPoints;
