@@ -17,18 +17,12 @@ GNU General Public License for more details.
 
 #include "pencilerror.h"
 #include <map>
-#include <QObject>
 #include <QSysInfo>
-#include "pencildef.h"
 
 DebugDetails::DebugDetails()
 {
 }
 
-DebugDetails::~DebugDetails()
-{
-}
-
 void DebugDetails::collect(const DebugDetails& d)
 {
     for (const QString& s : d.mDetails)
@@ -85,6 +79,18 @@ Status::Status(ErrorCode code)
 {
 }
 
+Status::Status(const ErrorCode code, const QString& description)
+    : mCode(code)
+    , mDescription(description)
+{
+}
+
+Status::Status(Status::ErrorCode eCode, const DebugDetails& detailsList)
+    : mCode(eCode)
+    , mDetails(detailsList)
+{
+}
+
 Status::Status(Status::ErrorCode eCode, const DebugDetails& detailsList, QString title, QString description)
     : mCode(eCode)
     , mTitle(title)
@@ -93,24 +99,18 @@ Status::Status(Status::ErrorCode eCode, const DebugDetails& detailsList, QString
 {
 }
 
-Status::Status(const ErrorCode code, const QString& title, const QString& description)
-    : mCode(code)
-    , mTitle(title)
-    , mDescription(description)
-{
-}
 
-QString Status::msg()
+QString Status::msg() const
 {
     static std::map<ErrorCode, QString> msgMap =
     {
         // error messages.
-        { OK,                    QObject::tr("Everything ok.") },
-        { FAIL,                  QObject::tr("Ooops, Something went wrong.") },
-        { FILE_NOT_FOUND,        QObject::tr("File doesn't exist.") },
-        { ERROR_FILE_CANNOT_OPEN,    QObject::tr("Cannot open file.") },
-        { ERROR_INVALID_XML_FILE,    QObject::tr("The file is not a valid xml document.") },
-        { ERROR_INVALID_PENCIL_FILE, QObject::tr("The file is not valid pencil document.") },
+        { OK,                    tr("Everything ok.") },
+        { FAIL,                  tr("Ooops, Something went wrong.") },
+        { FILE_NOT_FOUND,        tr("File doesn't exist.") },
+        { ERROR_FILE_CANNOT_OPEN,    tr("Cannot open file.") },
+        { ERROR_INVALID_XML_FILE,    tr("The file is not a valid xml document.") },
+        { ERROR_INVALID_PENCIL_FILE, tr("The file is not valid pencil document.") },
     };
 
     auto it = msgMap.find(mCode);
